#Davis Disassembly Main Code Open First

#This is a script that will execute all code needed to reproduce analysis for the Davis 2016 paper "What North America’s skeleton crew of megafauna tells us about community disassembly". For ease of human reading, this code will source code from other blocks in this directory.

#To run this code, keep all the code blocks and data table .csv's in the same folder and make sure that that folder is set as your working directory. 

#Then highlight all this code and hit run. In about 15 hours, all figures and tables used in the article will be generated and saved to this folder. 

#The algorithm that works with convex hulls takes a really long time. If you want to speed up the analysis considerably, just use fewer iterations for the null model. Right now it is set to 10,000 iterations.  

#Written by Matt Davis and adapted from his Quaternary Function Basefile 3.4.R on June 16, 2016
#matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This script has many redundancies because it was stiched together from several analysis scripts that were run one at a time. On the plus side, this makes it easier to run specific blocks of code without having to run the whole script. Note that this code generates data files that will be used in later portions of the code so you cannot always just skip ahead and try later analyses



##############################################################!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#Make sure your working directory is set to the folder where all these code blocks are located
#setwd()


##############################################################################################################################


#First and foremost, some bookkeeping
#This makes sure all the packages are loaded
if(!require("pacman")) install.packages("pacman")
pacman::p_load(ade4, boot, cluster, data.table, FD, geiger, geometry, ggplot2, grid, gridExtra, hier.part, Hmisc, labdsv, MuMIn, picante, plyr, reshape2, vegan,  update=F)

#And because this one isn't on CRAN
source("https://bioconductor.org/biocLite.R")
biocLite("ggtree")
#To avoid updating old packages
n


#Now on with the script!

#rm(list = ls())
############################################################





###
#FUNCTIONAL SPACE
###



#First get the working directory
mainDir <- getwd()
#All these getwd() are redundant but they are kept in for legacy reasons. 

#Show what the species functional tree and space look like 
require(picante)
require(Hmisc)	
require(cluster)


#Load up the traits
#This will take in trait data from a simplified csv version of Table S1 where the log 10 of sociality has already been performed. This file corresponds to Allrange 1.2 from Master species and traits 4.b.1 and Pleistocene Dates 5.3 and Pleistocene Synomomy 1.4.csv in the author's own version control system
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


#Pull out traits, caculate distance and create the tree

#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9,11:19)]
traits

#Set up the weights.  Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))


	
#Bookkeeping
traits <- as.matrix(traits)

#Traits Z transformed so they have a sd of 1 and mean of 0
ndf <- apply(traits, 2, scale, center = T, scale = T)

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2
rownames(traits2) <- rownames(allrange)

#############################

#Plot Functional dendrogram

#Make the distance matrix, don't want standardization because we already standardized and weighted
dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
#Now make the functional dendrogram
tree <- hclust(dist1,method="average")
plot(tree)


#Check the correlation between tree distance and Euclidean distance ~a measure of tree quality to capture all the data
treedist <- cophenetic(tree)
copheneticcorrelation <- cor(dist1, treedist)#.74

#To make a better version of that tree
ftree <- as.phylo(tree)
str(ftree)

require(ggplot2)
require(ggtree)
require(gridExtra)



fulltree <- ftree
#These are the tips to drop
prunetips <- allrange[allrange$Status=="Extinct","Labels"]

#But get the tips to keep
keeptips <- fulltree$tip.label[!fulltree$tip.label %in% prunetips]

#Group the tips to keep
prunetree <- groupOTU(fulltree, focus=keeptips)

#And plot
#First column of data has to match tiplabels of tree
colnames(allrange)
allrange2 <- cbind(allrange[,"Labels"],allrange)
colnames(allrange2)[1] <- "Taxa"
colnames(allrange2)

#Create base tree
p <- ggtree(prunetree,layout="circular", aes(color=group))
#Sync data
p <- p %<+% allrange2 

#Plot
p2 <- p+
scale_color_manual(values=c("lightgrey","black","dodgerblue2","red1","green4"))+
geom_tiplab(aes(subset=(abs(angle)<90),angle=angle,col=Status,label=Binomial), fontface="italic",size=3)+
geom_tiplab(aes(subset=(abs(angle)>=90),angle=angle+180,col=Status,label=Binomial), fontface="italic",size=3, hjust=1)+
xlim(0,0.65)
p2


ggsave(filename="Figure 1a.pdf")

###########################


#Plot PCA Trait space

#Do the pca, traits were already centered and scaled
pca1 <- prcomp(traits2,center=F,scale=F)


origscore <- pca1$x


#And Figure out the proportion of variance
eig1 <- round((summary(pca1)$importance[2,1])*100,digits=0)
eig2 <- round((summary(pca1)$importance[2,2])*100,digits=0)
eig3 <- round((summary(pca1)$importance[2,3])*100,digits=0)
eig4 <- round((summary(pca1)$importance[2,4])*100,digits=0)
#Explains 72, 7, 6, 5 % of variance
eig1+eig2+eig3+eig4 #Total 90%


#Checking loadings
require(vegan)
ef <- envfit(origscore, traits2, permu=999)
ef

str(ef)

#                   PC1      PC2     r2 Pr(>r)    
#Log10Massg    -0.99060 -0.13682 0.9983  0.001 ***
#Invertebrates  0.66926 -0.74303 0.1058  0.007 ** 
#Vertebrates    0.15533 -0.98786 0.8557  0.001 ***
#Graze         -0.99997  0.00758 0.1692  0.001 ***
#Browse        -0.08440  0.99643 0.8474  0.001 ***
#Running        0.05408 -0.99854 0.1064  0.011 *  
#Digging        0.99984 -0.01791 0.1230  0.005 ** 
#Climbing       0.60705 -0.79467 0.3063  0.001 ***
#Swimming      -0.47260 -0.88128 0.0669  0.048 *  
#Sociality     -0.40683  0.91350 0.1271  0.003 ** 





#Plot version with loadings
pdf(width=10,height=7.5, file="Figure S21.pdf")
plot(origscore[,1:2], type="n",xlab=paste("PC1 (",eig1,"% of variance)",sep=""),ylab=paste("PC2 (",eig2,"% of variance)",sep=""))
text(origscore[,1:2], rownames(origscore), cex=0.3, font=3)
plot(ef, col="red")
dev.off()



#Get native and extant species
natives <- origscore[allrange$FAD>49000,1:2]
modern <- origscore[allrange$Status=="Extant",1:2]

#Get the convex hull values
nativehull <- chull(natives)
nativehull <- c(nativehull,nativehull[1])

modernhull <- chull(modern)
modernhull <- c(modernhull,modernhull[1])


#Plot PCA Space with convex hulls
pdf(width=6,height=6, file="Figure 1b.pdf")

plot(origscore[,1:2],type="n",xlab=paste("PC1 (",eig1,"% of variance)",sep=""),ylab=paste("PC2 (",eig2,"% of variance)",sep=""))

#Plot convex hulls
lines(natives[nativehull,1:2],col="#ff7777",lwd=3.4)
lines(modern[modernhull,1:2],col="lightskyblue",lwd=4.2)

#Plot points
points(origscore[allrange$Status=="Extinct",1:2],col="red1",pch=19)
points(origscore[allrange$Status=="Extant",1:2],col="dodgerblue2",pch=19)
points(origscore[allrange$Status=="Introduced",1:2],col="green4",pch=19)

dev.off()

#And a plot for the inset
pdf(width=6,height=6, file="Figure 1b inset.pdf")

plot(origscore[,1:2],type="n")
plot(ef, col="black")
dev.off()



rm(list = ls())
######################################################





###
#FUNCTIONAL DIVERISTY OVER TIME
###


#First get the working directory
mainDir <- getwd()

#Plot species' temporal ranges
#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#First run binning code
source(file.path(mainDir,"Davis Timebinning.R"), chdir = F)

#Pull out the fake bin that has all of the species in it
xbin <- allbin[allbin$Bin==max(unique(allbin$Bin)),]
dim(xbin)#94 35

alltaxa <- xbin$Binomial
numtaxa <- length(alltaxa)

#Make the occurence matrix
therematrix <- matrix(data=0, ncol=timesteps-1, nrow=numtaxa, dimnames=list(alltaxa,inoutdates[1:(timesteps-1)]))
#dim(therematrix)


#Now fill it up
#For each species
for(i in 1:numtaxa){
	
	thistaxa <- numtaxa[i]
	
	#Take out lads and fads
	FAD <- xbin[i,"FAD"]
	LAD <- xbin[i,"LAD"]
	
	#For each timestep
	for(j in 1:(timesteps-1)){
	
	#Enter 1 if taxa occurs in that bin.  Bin includes younger edge but not older edge
	therematrix[i,j]<- ifelse((FAD>=inoutdates[j] & LAD<inoutdates[j+1]),1,0)
	
		
		}#End j loop
}#End i loop

#Now plotting
require(ggplot2)
require(reshape2)

#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)

#Sort species by their lads first, then fads, and use this order to plot ggplot
sortrange <- allrange[order(allrange$LAD, allrange$FAD), ]
head(sortrange)

#Now order the matrix by that
therematrix
sortmatrix <- therematrix[match(sortrange$Binomial,rownames(therematrix)),]

#Melt the matrix into something ggplot can use
df <- melt(sortmatrix, varnames = c("Binomial", "Age"), value.name = "Present")
df$Present <- as.character(df$Present)

head(df)
tail(df)


####Now plot
ggplot(df, aes_string(x = "Age", y = "Binomial", fill = "Present"))+
geom_raster(hjust=0) + #This lines up the tiles                     
scale_x_reverse(name="Years before present")+
scale_y_discrete(name="Species")+
scale_fill_manual(values=c("1"="black","0"="white")) +
ggtitle("Temporal ranges")+
simpleblack+
theme(legend.position="none", axis.text.y=element_text(size=6, face="italic"),axis.text.x=element_text(size=14))

ggsave(filename="Figure S1.pdf",width=10,height=7.5)
ggsave(filename="Figure S1.png",width=10,height=7.5)

#And a vertical version
#ggsave(filename="Figure S1 vertical.png",width=6,height=7)


#If this plot appears blurry in pdf form, check to make sure your computer is not automatically doing pdf smoothing

rm(list = ls())
##########################


#First get the working directory
mainDir <- getwd()

#Load up data and calculate functional diversity over time
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Need to set new times for possible future extinctions
#By default, endangered species will get a LAD of 0
#Give a LAD of -500 (500 years in the future) to those species that are near threatened
allrange[allrange$Red.List.status=="NT",32] <- -500

#And give a LAD of -1500 to those species that aren't threatened
allrange[allrange$Red.List.status=="DD"| allrange$Red.List.status=="LC",32] <- -1500



#Pull out the traits and dates
colnames(allrange)
traits <- allrange[,c(9,11:19)]
dates <- allrange[31:32]

head(traits)
head(dates)
dim(traits)#94 10
dim(dates)#94 2

#Set the datatype
datatype <- "Function results Allrange 1.2 equal weights quickfricfdladfad12"

#Set up the weights
#Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))

source(file=file.path(mainDir,"Davis Functional Diversity.R"), chdir = F)


starttime <- proc.time()
results<- quickfricfdladfad12(traits=traits,dates=dates, weights=weights,datatype=datatype)
Totaltime <- (proc.time()-starttime)
Totaltime
#Takes a few seconds

#Check the results
str(results)
save(results, file="Function results")

rm(list = ls())
######################################################



#Run the null model to determine signficance of actual extinction scenario. This null model has random extinctions but introductions of humans and domestic animals are at actual dates

#First get the working directory
mainDir <- getwd()

#Load in results of actual extinciton
load(file=file.path(mainDir,"Function results"))
str(results)

#And allrange
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


source(file.path(mainDir,"Davis Functional Diversity Simulation.R"), chdir = F)

#Pull out the results of the actual extinction
realdata1 <- results$data1

#And make the traits and weights and dates
traits <- results$traits
dim(traits)#94 10
weights <- results$weights
dates <- results$dates


##################
#Now run the null model
#starttime <- proc.time()

#simresults<- simfd6(traits=traits, weights=weights,realdata1=realdata1,dates=dates, n=1000)

#Totaltime <- (proc.time()-starttime)
#Totaltime

#Totaltime/60
#1000 reps takes about 26 minutes


#Check the results
#str(simresults, max.level=1)
#save(simresults, file="Functional Simulation Results 1000")


##########
#For 10,000 reps

#Now run the null model
starttime <- proc.time()

simresults<- simfd6(traits=traits, weights=weights,realdata1=realdata1,dates=dates, n=10000)

Totaltime <- (proc.time()-starttime)
Totaltime

Totaltime/60
#10000 reps takes about 4.5 hours 


#Check the results
str(simresults, max.level=1)
save(simresults, file="Functional Simulation Results 10000")



rm(list = ls())
######################################################



#Plot what the new null model produced

#First get the working directory
mainDir <- getwd()

#Load up the results
load(file=file.path(mainDir,"Functional Simulation Results 10000"))
str(simresults,max.level=1)

#Load in real results too
#Load in results of actual extinciton
load(file=file.path(mainDir,"Function results"))
str(results)


#Now plot. First some data wrangling

require(ggplot2)
require(plyr)
require(reshape2)
require(gridExtra)
require(data.table)

#Pull out the simulation list and turn it into a dataframe
simlist <- simresults$simlist
head(simlist)

#Use rbindlist as it is much much faster than do.call(rbind)
simdf <- as.data.frame(rbindlist(simlist))
head(simdf)
tail(simdf)
class(simdf)

#Test plot
#ggplot(data=simdf, aes(x=RelRich,y=FD,group=Iteration))+geom_line(alpha=0.04)

#ggplot(data=simdf, aes(x=RelRich,y=FRic,group=Iteration))+geom_line(alpha=0.04)


#Calculate the confidence intervals

#First have to melt the data frame
meltsim <- melt(simdf,id.vars=c("RelRich","Iteration","Bin","Date"))
head(meltsim)
colnames(meltsim)[5:6] <- c("Index","Value")
head(meltsim)
tail(meltsim)

#figure out medians and 95% confidence intervals for all the functional indices
indexsum <- ddply(meltsim,.(RelRich, Bin, Date, Index), summarise,Median=quantile(Value, 0.5,na.rm=T),CIl95=quantile(Value,0.025,na.rm=T),CIu95=quantile(Value, 0.975,na.rm=T))
dim(indexsum)#972 7

indices <- sort(unique(meltsim$Index))

#Trim out last bin because it is a fake bin with all species in them
indexsum2 <- subset(indexsum, Bin!=max(indexsum$Bin))
dim(indexsum2)#954 7

###################

#Now prep the real data
data1 <- results$data1

head(data1)
tail(data1)
#Add relative richness category 
data1$RelRich <- data1$SpeciesRichness/max(data1$SpeciesRichness)

#And get the real trait means
traitmeans <- simresults$realtraitmeans

#hook them together
data2 <- cbind(data1,traitmeans)
head(data2)
tail(data2)

#Delete last row because it is the fake bin
data2 <- data2[-dim(data2)[1],]

#Melt the prediction data
meltdata <- melt(data2,id.vars=c("RelRich","Bin","Date"))
head(meltdata)
colnames(meltdata)[4:5] <- c("Index","Value")
head(meltdata)
tail(meltdata)

#Split this into just actual data and future extinctions too

meltreal <- subset(meltdata, Date>=0)
dim(meltreal)#750 5
head(meltreal)
tail(meltreal)


#######################
#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)


#Subset simulated data to just species richness functional diversity indices
funcindexsum <- subset(indexsum2,Index %in% c("SpeciesRichness","FD","FRic","FDis","RaoQ"))

#Subset real data to just functional diversity indices
#Without future extincitons
funcmeltreal <- subset(meltreal,Index %in% c("FD","FRic","FDis","RaoQ"))
#With future extinctions
funcmeltdata <- subset(meltdata,Index %in% c("FD","FRic","FDis","RaoQ"))



ggplot(data=funcindexsum)+
#Plot null expecations
geom_line(aes(x=Date,y=Median,color=Index),size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95,fill=Index),alpha=.15)+
geom_line(aes(x=Date,y=CIl95,color=Index),size=0.5)+
geom_line(aes(x=Date,y=CIu95,color=Index),size=0.5)+

#Plot the real results
geom_line(data=funcmeltdata,aes(x=Date,y=Value,group=Index),color="black",size=1)+

#Set up colors to match previous graphs
scale_color_manual(name="Index",values=c("#e41a1c","#ff7f00","#377eb8","#4daf4a","#984ea3"))+
scale_fill_manual(name="Index",values=c("#e41a1c","#ff7f00","#377eb8","#4daf4a","#984ea3"))+

facet_grid(Index ~., scales="free_y")+
scale_x_reverse(name="Years before present")+
scale_y_continuous(name="Index value")+
theme(legend.position="none")+
#Remove labels
theme(strip.background=element_blank(),strip.text=element_blank())+

#labs(title="Functional index values over time\nColored lines represent median and 95% CI from 1000 simulation runs")+
simpleblack
ggsave(filename="Figure 2.pdf",width=82,units="mm")





###########################


#Plot dendogram based Sorensen index of similarity

ggplot(data=subset(indexsum2,Index=="FDsim"))+
#Plot null expecations
geom_line(aes(x=Date,y=Median),color="darkcyan",size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95),fill="darkcyan",alpha=.15)+
geom_line(aes(x=Date,y=CIl95),color="darkcyan",size=0.5)+
geom_line(aes(x=Date,y=CIu95),color="darkcyan",size=0.5)+

scale_x_reverse(name="Years before present")+
scale_y_continuous(name="Sørensen index of similarity")+
theme(legend.position="none")+
#labs(title="Sørensen index of functional similarity\nSimulated communities compared to actual extinction\nColored lines represent median and 95% CI from 1000 simulation runs")+
simpleblack
#+presentmod

ggsave(filename="Figure S13.pdf",width=6,height=4.5)



##########################


#Now plot null expectations for traits

#Diet first

ggplot(data=subset(indexsum2,Index %in% c("Browse","Graze","Vertebrates","Invertebrates")))+
geom_line(aes(x=Date,y=Median,color=Index),size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95,fill=Index),alpha=.15)+
geom_line(aes(x=Date,y=CIl95,color=Index),size=0.5)+
geom_line(aes(x=Date,y=CIu95,color=Index),size=0.5)+

#Plot the real results
geom_line(data=subset(meltdata,Index %in% c("Browse","Graze","Vertebrates","Invertebrates")),aes(x=Date,y=Value,group=Index,color=Index),size=1.5,linetype=3)+

#Formatting
scale_y_continuous(name="Percent of diet")+
scale_x_reverse(name="Years before present")+
#ggtitle("Mean diet through time\nColored lines represent median and 95% CI from 1000 simulation runs\nDotted line is real value")+
simpleblack
#+presentmod#+
#To put the legend on the bottom
#theme(legend.position="bottom")

ggsave(filename="Figure S18.pdf",width=6,height=4.5)

#######


#Now for locomotor traits

ggplot(data=subset(indexsum2,Index %in% c("Running", "Digging", "Climbing", "Swimming")))+
geom_line(aes(x=Date,y=Median,color=Index),size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95,fill=Index),alpha=.15)+
geom_line(aes(x=Date,y=CIl95,color=Index),size=0.5)+
geom_line(aes(x=Date,y=CIu95,color=Index),size=0.5)+

#Plot the real results
geom_line(data=subset(meltdata,Index %in% c("Running", "Digging", "Climbing", "Swimming")),aes(x=Date,y=Value,group=Index,color=Index),linetype=3,size=1.5)+

#Formatting
scale_y_continuous(name="Locomotor level")+
scale_x_reverse(name="Years before present")+
#ggtitle("Mean locomotor values through time\nColored lines represent median and 95% CI from 1000 simulation runs\nDotted line is real value")+
simpleblack
#+presentmod
#theme(legend.position="bottom")

#climbing and digging really increase with extinction

ggsave(filename="Figure S17.pdf",width=6,height=4.5)


###########


#Mean weight
ggplot(data=subset(indexsum2,Index=="Log10Massg"))+
geom_line(aes(x=Date,y=Median,color=Index),size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95,fill=Index),alpha=.15)+
geom_line(aes(x=Date,y=CIl95,color=Index),size=0.5)+
geom_line(aes(x=Date,y=CIu95,color=Index),size=0.5)+

#Plot the real results
geom_line(data=subset(meltdata,Index=="Log10Massg"),aes(x=Date,y=Value,group=Index,color=Index),linetype=3,size=1.5)+

scale_x_reverse(name="Years before present")+
scale_y_continuous(name="Average mass (log grams)")+
#ggtitle("Mean body mass through time\nColored lines represent median and 95% CI from 1000 simulation runs\nDotted line is real value")+
theme(legend.position="none")+
simpleblack
#+presentmod

ggsave(filename="Figure S19.pdf",width=6,height=4.5)


########


#For sociality
ggplot(data=subset(indexsum2,Index=="Sociality"))+
geom_line(aes(x=Date,y=Median,color=Index),size=1)+
geom_ribbon(aes(x=Date,ymin=CIl95,ymax=CIu95,fill=Index),alpha=.15)+
geom_line(aes(x=Date,y=CIl95,color=Index),size=0.5)+
geom_line(aes(x=Date,y=CIu95,color=Index),size=0.5)+

#Plot the real results
geom_line(data=subset(meltdata,Index=="Sociality"),aes(x=Date,y=Value,group=Index,color=Index),linetype=3,size=1.5)+

scale_x_reverse(name="Years before present")+
scale_y_continuous(name="Average sociality (log number of individuals in group)")+
#ggtitle("Mean group size through time\nColored lines represent median and 95% CI from 1000 simulation runs\nDotted line is real value")+
theme(legend.position="none")+
simpleblack
#+presentmod


ggsave(filename="Figure S20.pdf",width=6,height=4.5)


##########################


#Print actual results of extinction
#Pull out bins for future, present, precolumbian, and start
datasum <- rbind(data1[c(1,3,4,53),])
datasum$RelRichness <- datasum$SpeciesRichness/max(datasum$SpeciesRichness)
round(datasum,digits=2)


##########################
#Compare actual results to null to find out where there is a significant difference

#!Note these exact values may be different given that the null model is performing random extinctions

#Merge results with simulated data
msum <- merge(meltdata,indexsum2, by=c("RelRich","Index","Date","Bin"),all.x=T)
dim(msum) #780 8
head(msum)
tail(msum)
msum$Below <- msum$Value-msum$CIl95

msum[msum$Below<0 & msum$Index=="FD",]
#FD drops below at 9000 and 3000,2000,1000, and -1000 but not 0. Value is never more than .5% below
msum[msum$Below<0 & msum$Index=="FDis",]
#FDis drops below at 9000 to 1000, and then -2000, never more than 1% below
msum[msum$Below<0 & msum$Index=="RaoQ",]
#RaoQ drops below at 9000 to -2000, sometimes over 1%

rm(list = ls())
#####################################################


#April 4

#EFFECT AND RESPONSE TRAITS

#Show if effect traits (Functional distinctiveness) are correlated with response traits (extinction risk)

#########################


#First need to carry out a simulation of random extinction of just native taxa

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Remove all species that enter the system
allrange2 <- allrange[allrange$FAD>49000,]
dim(allrange2)#85 33 checks out
colnames(allrange2)

traits <- allrange2[,c(9,11:19)]
dim(traits)#85 10

#Set up the weights outside the function call.
#Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))


source(file.path(mainDir,"Davis Functional Diversity Random Simulation.R"), chdir = F)


starttime <- proc.time()

simresults<- simfd5(traits=traits, weights=weights, n=10000)

Totaltime <- (proc.time()-starttime)
Totaltime

Totaltime/3600
#10000 reps takes about 1.5 hours


#Check the results
str(simresults)
save(simresults, file="Functional Random Simulation Results 10000")

#These will be used for plotting purposes later

rm(list = ls())
######################################################

#Performing a logistic regression to see what traits are most correlated with extinction

#Load up the traits
#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


#Remove all species that enter the system by selecting natives
allrange2 <- allrange[allrange$FAD>49000,]
dim(allrange2)#85 33 checks out
colnames(allrange2)

traits <- allrange2[,c(9,11:19)]
dim(traits)#85 10


#Do a hierarchical partitioning to find out which variables are most important  
require(hier.part)  
statusv <- ifelse(allrange2$Status=="Extinct",0,1)
predictors <-traits
class(statusv)

hfit <- hier.part(y=statusv, xcan=predictors, family="binomial", gof="logLik",barplot=T)
summary(hfit)

#This takes a few seconds
rand.hp(y=statusv, xcan=predictors, family="binomial", gof="logLik",num.reps=20)

#                Obs Z.score sig95
#Log10Massg     8.85   18.65     *
#Invertebrates  0.12   -0.32      
#Vertebrates    0.48    0.17      
#Graze          0.39    0.49      
#Browse         0.11   -0.28      
#Running        0.07   -0.60      
#Digging        0.98    2.38     *
#Climbing       2.81    6.55     *
#Swimming      -0.12   -1.16      
#Sociality      0.56    0.03      

  
#So it is looking like mass, running, digging and climbing are all potentially important

#Now perform logistic regression
predictors$Status <- allrange2$Status
predictors <- droplevels(predictors)

fitmdc <- glm(Status~ Log10Massg + Digging +Climbing,data=predictors,family=binomial())
summary(fitmdc)

fitmc <- glm(Status~ Log10Massg +Climbing,data=predictors,family=binomial())
summary(fitmc)

fitm <- glm(Status~ Log10Massg,data=predictors,family=binomial())
summary(fitm)


#Compare models using AICc
require(MuMIn)
require(geiger)

aicc <- AICc(fitmc,fitmdc,fitm)

round(aicw(aicc$AICc),digits=2)
#So mass/climbing is best, then mass, climbing and digging, then just mass

#    fit delta    w
#mc 76.09  0.00 0.46
#mdc 76.76  0.68 0.33
#m 77.63  1.54 0.21




round(exp(coef(fitmc)),digits=2)
exp(confint(fitmc))
#So for mass and climbing, each time mass multiplies by 10, risk of extinction goes up by 11, each time climbing goes up by 1, extinction risk halves

#To get probabilities of extinction given the mass climbing model
mcp <- predict(fitmc,predictors,type="response")

#Try cross validating model with cv.glm kfold cross validation so dataset is split into 10 sets, one used for testing, 9 used for training
require(boot)

cv.glm(data=predictors,glmfit=fitmc,K=dim(predictors)[1])
#Prediction error is 0.149 for fitmc
cv.glm(data=predictors,glmfit=fitmdc,K=dim(predictors)[1])
#Prediction error is 0.152 for fitmdc
cv.glm(data=predictors,glmfit=fitm,K=dim(predictors)[1])
#Prediction error is 0.151 for fitm


#Add probabilites from best model to dataframe
allrange2$Probmc <- mcp
head(allrange2)

#########


#Ok, now to get functional distinctiveness and actual extinction order


#Can use uniquebranchlengths code but it takes weights and allrange so have to provide
allrange <- allrange2
#Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))

#This function returns functional distinctiveness and uniqueness for all the species in each bin. For now it is just being used to get the bin that each species goes extinct in
#This takes a few seconds
source(file.path(mainDir,"Davis Unique Branchlengths.R"), chdir = F)
dim(allbin)#3808 39
head(allbin)
tail(allbin)


#Get the earliest real bin
startbin <- allbin[allbin$Bin==52,]
dim(startbin)#85 39

#Rank species by functional distinctiveness
startbin[order(startbin$RelFD,decreasing=T),]

sort(round(startbin[startbin$Genus=="Mammuthus","RelFD"]*100,digits=2))
#Mammoths have functional distinctiveness going from 1.06-2.30%

sort(round(startbin[startbin$Genus=="Nasua","RelFD"]*100,digits=2))
#So Nasua narica is 2.16%

#########################
#Quickly plot functional uniqueness between extinct and extant species 

require(ggplot2)

#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)



test1 <- t.test(RelUniq~Status, data=startbin)

#p=0.86 distributions aren't different


ggplot(data=startbin,aes(x=Status,y=100*RelUniq,fill=Status))+
geom_boxplot(size=2)+
scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test1$p.value,digits=2),sep=""))+

#Formatting
scale_y_continuous(name="Dendrogram functional uniqueness")+
#ggtitle("Extinct species are no more functionally unique than extant species")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

ggsave(filename="Figure S7.pdf",width=6,height=4.5)
###########################



#Continue on with getting bins where species go extinct

exbin <- allbin[allbin$ExtinctBin=="Going extinct",]
#But remove future extinctions
exbin2 <- exbin[exbin$Age>0,]
dim(exbin2)#42 39
dim(allrange2[allrange2$Status=="Extinct",])#42 34 checks out
exbin2

#Now put those extinction bins and ages into startbin
mbin <- data.frame(exbin2$Binomial,exbin2$Bin,exbin2$Age)
colnames(mbin) <- c("Binomial","ExtinctBinNum","ExtinctAge")

startbin2 <- merge(startbin,mbin,by="Binomial", all.x=T)
head(startbin2)

####################
#Now plot against each other

#Plot probability of extinction vs functional distinctiveness
ggplot(data=startbin2, aes(x=Probmc,y=RelFD*100,group=Status,color=Status))+

#Now do each regression line
#stat_smooth(method="lm",data=subset(allrange2,Status=="Extant"),colour="dodgerblue2",fill="dodgerblue2",size=3)+
#stat_smooth(method="lm",data=subset(allrange2,Status=="Extinct"),colour="red1",fill="red1",size=3)+
stat_smooth(method="lm",aes(group=1),color="grey",size=3)+

#Actual points
geom_point(size=4)+
scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#Formatting
scale_x_reverse(name="Probability of extinction\n(Response trait)")+
scale_y_continuous(name="Functional distinctiveness\n(Effect trait)")+
#ggtitle("")+
simpleblack+
#presentmod+
theme(legend.title=element_blank())
#legend.position="bottom",

ggsave(filename="Figure S4.pdf",width=6,height=4.5)



#######
#Now plot the actual extinction order 
#Hack plot to put distributions in 
#Some modelling up top
extantFD <- startbin2[startbin2$Status=="Extant","RelFD"]*100
extinctFD <- startbin2[startbin2$Status=="Extinct","RelFD"]*100

t.test(startbin2$RelFD~startbin2$Status)$p.value#0.79

fit1<-lm(RelFD*100~ExtinctAge, data=startbin2) 

#Make a fake dataset
newx<-seq(min(startbin2$ExtinctAge,na.rm=T),max(startbin2$ExtinctAge,na.rm=T),length.out=100) 

newx <- data.frame(x=newx)
colnames(newx) <- "ExtinctAge"
head(newx)

prd<-predict(fit1,newdata=newx,interval = c("confidence"), level = 0.95) 




#Plot points first
#bottom left top right


pdf(width=6.14, height=3, pointsize=10,file="Figure 3.pdf")
par(mar=c(5.1 ,5, 0.5, 0.5))

plot(x=startbin2$ExtinctAge, y=startbin2$RelFD*100, col="#ff7777",type="n",xlim=c(55000,0), ylab="Functional distinctiveness\n(Effect trait)",xlab="",xaxt="n",pch=19)

#Set clipping box x1,x2,y1,y2
#Save out clip first
usr <- par("usr")
clip(min(startbin2$ExtinctAge,na.rm=T),max(startbin2$ExtinctAge,na.rm=T),min(startbin2$RelFD*100,na.rm=T),max(startbin2$RelFD*100,na.rm=T))
#Linear regression
abline(reg=fit1,col="red1") 
#Then confidence interval
lines(x=newx$ExtinctAge,y=prd[,2],col="red1",lty=2) 
lines(x=newx$ExtinctAge,y=prd[,3],col="red1",lty=2) 

#Add the points in
clip(usr[1],usr[2], usr[3], usr[4])
points(x=startbin2$ExtinctAge, y=startbin2$RelFD*100, col="#ff7777",pch=19,cex=0.85)


#Then boxplots
boxplot(extantFD,extinctFD, add=T, boxwex=4000,at=c(53000,47000),col=c("lightskyblue","#ff7777"),lwd=1.5)

#Now add in x axis
axis(side=1, at=c(53000,47000,40000,30000,20000,10000,0),labels=c("Extant","Extinct","40000","30000","20000","10000","0"))


#And some labels
mtext("Date of extinction\n(Response trait)",side=1,at=20000,line=3.5)
mtext("(a)",side=3,at=55750,line=-1.25,font=3)
mtext("(b)",side=3,at=41300,line=-1.25,font=3)

#Separation line
abline(v=43000)


dev.off()




#########################


#Simulation of extinction

#So now need to compare what actually happened to functional diversity with best and worst case scenarios

#Will need to create 4 there matrices, best, case, worst case, actual extinction, extinction probabilites

#Create the normal species timebin occurence matrix first

#Bookkeeping
alltaxa <- rownames(allrange2)
numtaxa <- length(alltaxa)
dates <- allrange2[,31:32]
###


#For equal time bins not including possible future extinctions
inoutdates1 <-seq(0,50000,by=1000)
timesteps <- length(inoutdates1)
bins <- inoutdates1
	
#Make the occurence matrix
therematrix <- matrix(data=0, nrow=timesteps, ncol=numtaxa, dimnames=list(c(inoutdates1),rownames(dates)))
	

	#Now fill it up
	#For each species
	for(i in 1:numtaxa){
	
	thistaxa <- alltaxa[i]
	
	#Take out lads and fads
	FAD <- dates[i,1]
	LAD <- dates[i,2]
	
		#For each timestep
		for(j in 1:timesteps){
	
			#And fill up only that taxa's column 
			therematrix[j,thistaxa] <- 	ifelse((FAD>=bins[j] & LAD<bins[j+1]),1,0)


			
		}#End j loop
	}#End i loop

#Remove fake last bin and rename
actualtherematrix <- therematrix[-dim(therematrix)[1],]


##########################
#Source in the therematrix creator function
source(file.path(mainDir,"Davis Occurrence Matrix.R"), chdir = F)

###########################
#Now for a conservation plan based on branch based functional distinctiveness
head(startbin2)

#Reorder by functional distinctiveness
bestdistinct <- startbin2[order(startbin2$RelFD,decreasing=F), "Labels"]

head(bestdistinct)

bestcasefunctdistincttherematrix <- createtherematrix(alltaxa=alltaxa,taxonorder=bestdistinct)
dim(bestcasefunctdistincttherematrix)#85 85	

#Now worst case
worstdistinct <- startbin2[order(startbin2$RelFD,decreasing=T), "Labels"]
head(worstdistinct)

worstcasefunctdistincttherematrix <- createtherematrix(alltaxa=alltaxa,taxonorder=worstdistinct)
dim(worstcasefunctdistincttherematrix)#85 85	
head(worstcasefunctdistincttherematrix)

######################

#Now for extinctions based on extinction probabilities
head(startbin2)
probex <- startbin2[order(startbin2$Probmc,decreasing=T), "Labels"]
head(probex)

probextincttherematrix <- createtherematrix(alltaxa=alltaxa,taxonorder=probex)
dim(probextincttherematrix)#85 85	
head(probextincttherematrix)


###############################################	


#Now run actual FD FRic analysis


#Load up code to run FD and FRic
source(file.path(mainDir,"Davis Functional Diversity Simple.R"), chdir = F)

#Load up simulation results from earlier
load(file.path(mainDir,"Functional Random Simulation Results 10000"))
str(simresults,max.level=1)

##############


#Now on to actually calculating the indices

#Create list for results
scenarios <- list()

actualextinction <- simplefd(traits2=simresults$traits2,pctraits=simresults$pctraits,therematrix=actualtherematrix, scenario="Actual extinction")

#And put it in a list
scenarios[[1]] <- actualextinction

###Best case functional distinctiveness
bestcase <- simplefd(traits2=simresults$traits2,pctraits=simresults$pctraits,therematrix=bestcasefunctdistincttherematrix, scenario="Best case/Negative response-effect correlation")

#And put it in a list
scenarios[[2]] <- bestcase


###Worst case functional distinctiveness
worstcase <- simplefd(traits2=simresults$traits2,pctraits=simresults$pctraits,therematrix=worstcasefunctdistincttherematrix, scenario="Worst case/Positive response-effect correlation")

#And put it in a list
scenarios[[3]] <- worstcase

###Best case functional distinctiveness
probexcase <- simplefd(traits2=simresults$traits2,pctraits=simresults$pctraits,therematrix=probextincttherematrix, scenario="Extinction prone species die first")

#And put it in a list
scenarios[[4]] <- probexcase


#Save all of the scenarios
save(scenarios, file="Functional Extinction Scenarios")



rm(list = ls())
#########################



#Load up the results and plot them

#First get the working directory
mainDir <- getwd()

#Load up random simulation results
load(file.path(mainDir,"Functional Random Simulation Results 10000"))
str(simresults,max.level=1)


#And conservation scenarios
load(file.path(mainDir,"Functional Extinction Scenarios"))
str(scenarios)




#First take the 1000 random extinctions and plot them up
require(ggplot2)
require(plyr)
require(reshape2)
require(data.table)
require(gridExtra)

#Turn the simulation list into a dataframe
#Use rbindlist as it is much much faster than do.call(rbind)
simdf <- as.data.frame(rbindlist(simresults$simlist))
head(simdf)
tail(simdf)
class(simdf)




#Calculate the confidence intervals

#First have to melt the data frame
meltsim <- melt(simdf,id.vars=c("SpeciesRichness","RelRich","Iteration"))
head(meltsim)
colnames(meltsim)[4:5] <- c("Index","Value")
head(meltsim)
tail(meltsim)

#figure out median and 95% confidence intervals for all the functional indices
indexsum <- ddply(meltsim,.(SpeciesRichness,RelRich, Index), summarise,Median=median(Value, na.rm=T),CIl95=quantile(Value, c(0.025, 0.975),na.rm=T)[1],CIu95=quantile(Value, c(0.025, 0.975),na.rm=T)[2])

indexsum

#Trim to just reasonably calulated simulated communites
indexsum2 <- subset(indexsum, SpeciesRichness>9)


####
#Now take the scenarios and turn them into a dataframe
scendf <- as.data.frame(do.call("rbind",scenarios))
head(scendf)
tail(scendf)

#Melt the data frame for ggplot
meltscen <- melt(scendf,id.vars=c("SpeciesRichness","RelRich","Scenario"))
head(meltscen)
colnames(meltscen)[4:5] <- c("Index","Value")
head(meltscen)
tail(meltscen)
dim(meltscen)#610 5

#Trim to just reasonably calulated simulated communites
meltscen2 <- subset(meltscen, SpeciesRichness>9)

head(meltscen2)
dim(meltscen2)#556 5


#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)



######Now FD


indexsumFD <- subset(indexsum2,Index=="FD")
meltscenFD <- subset(meltscen2,Index=="FD")

ggplot()+
geom_line(data=indexsumFD, aes(x=RelRich*100,y=Median*100),size=1.5,color="grey")+
geom_ribbon(data=indexsumFD,aes(x=RelRich*100, ymin=CIl95*100,ymax=CIu95*100),alpha=.15,fill="grey")+	
#Put in the real data
geom_path(data=meltscenFD,aes(x=RelRich*100,y=Value*100,group=Scenario,color=Scenario),size=2)+

scale_y_continuous(name="Percentage FD remaining")+
scale_x_continuous(name="Percentage of species remaining")+
coord_fixed()+
#labs(title="Hold")+
theme(plot.title=element_text(hjust=.5),legend.position="right")+
simpleblack
#+presentmod

ggsave(filename="Figure S2.pdf",width=6,height=4.5)


######Now FRic


indexsumFRic <- subset(indexsum2,Index=="FRic")
meltscenFRic <- subset(meltscen2,Index=="FRic")

ggplot()+
geom_line(data=indexsumFRic, aes(x=RelRich*100,y=Median*100),size=1.5,color="grey")+
geom_ribbon(data=indexsumFRic,aes(x=RelRich*100, ymin=CIl95*100,ymax=CIu95*100),alpha=.15,fill="grey")+	
#Put in the real data
geom_path(data=meltscenFRic,aes(x=RelRich*100,y=Value*100,group=Scenario,color=Scenario),size=2)+

scale_y_continuous(name="Percentage FRic remaining")+
scale_x_continuous(name="Percentage of species remaining")+
coord_fixed()+
#labs(title="Hold")+
theme(plot.title=element_text(hjust=.5),legend.position="right")+
simpleblack
#+presentmod

ggsave(filename="Figure S3.pdf",width=6,height=4.5)


rm(list = ls())
######################################################



###
#DIFFERENCES IN FUNCTIONAL SPACE BETWEEN EXTANT AND EXTINCT SPECIES
###


#Show if effect traits (Functional distinctiveness) are correlated with response traits (extinction risk)

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Take only native species
allrange4 <- allrange[allrange$FAD>49000,]
dim(allrange4)# 85 33

require(ggplot2)

#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)



test1 <- t.test(Log10Massg~Status, data=allrange4)
#p-value = 3.055e-11

#Plot up mass
ggplot(data=allrange4,aes(x=Status,y=Log10Massg, fill=Status))+
geom_boxplot(size=2)+
scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test1$p.value,digits=10),sep=""))+

#Formatting
scale_y_continuous(name="Log10 mass in g")+
#ggtitle("Extinct species were more massive")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

ggsave(filename="Figure S16.pdf",width=6,height=4.5)


########
#Plot up climbing
test2 <- t.test(Climbing~Status, data=allrange4)
#p-value = 4.135211e-05


ggplot(data=allrange4,aes(x=Status,y=Climbing, fill=Status))+
geom_boxplot(size=2)+
scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test2$p.value,digits=5),sep=""))+

#Formatting
scale_y_continuous(name="Climbing ability")+
#ggtitle("Extinct species were worse climbers")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

#ggsave(filename="Figure Extra1.pdf",width=6,height=4.5)


################
#Plot up climbing ability vs mass
ggplot(data=allrange4,aes(x=Log10Massg,y=Climbing,group=Status, color=Status))+

#Now do each regression line
stat_smooth(method="lm",data=subset(allrange4,Status=="Extant"),colour="dodgerblue2",fill="dodgerblue2",size=3)+
stat_smooth(method="lm",data=subset(allrange4,Status=="Extinct"),colour="red1",fill="red1",size=3)+
#stat_smooth(method="lm",aes(group=1),color="grey",size=4)+

#Add points
geom_point(size=4)+
scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_x_continuous(name="Log10 mass in g")+
scale_y_continuous(name="Climbing ability")+
#ggtitle("Extinct species were poor climbers regardless of size")+
simpleblack+
#presentmod+

theme(legend.title=element_blank())
#legend.position="bottom",

#ggsave(filename="Figure Extra2.pdf",width=6,height=4.5)


rm(list = ls())
########################



#Now look at measures of functional space

#Start calculating nearest neighbor distances in PCA space

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Take only native species
allrange <- allrange[allrange$FAD>49000,]
dim(allrange)# 85 33

#Also load in functional results to get correct number of axes to keep and weights
load(file.path(mainDir,"Function Results"))
str(results)
m <- results[["numaxes"]]
weights <- results[["weights"]]

#Run the nearestneighbor1.4 function
#This will calculate nearest neighbor distance for each taxa in each time bin but we will only use the earliest bin
source(file.path(mainDir,"Davis Nearest Neighbor.R"), chdir = F)
#Takes a few seconds

head(allbin)
tail(allbin)

#This is the earliest bin
xbin2 <- allbin[allbin$Bin==52,]
dim(xbin2)#85 25

#So for some context
max(dist1)#1.37 is max distance
which(dist1==max(dist1),arr.ind=T)
xbin2[xbin2$Binomial=="Alces americanus","DistCent"]/max(dist1)#.20
#This is between a Columbian mammoth and the Eira.  So a moose is about a fifth of that distance from the centroid right now


#Now plot
require(ggplot2)
#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)


test1 <- t.test(Dist1~Status, data=xbin2)

#p=0.43 distributions aren't different

###Plot distance to nearest neighbor
ggplot(data=xbin2,aes(x=Status,y=Dist1,fill=Status))+
geom_boxplot(lwd=2)+

scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test1$p.value,digits=2),sep=""))+

#Formatting
scale_y_continuous(name="Functional distance to nearest neighbor")+
#ggtitle("Distance to nearest neighbor")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

#ggsave(filename="Figure S Extra.pdf",width=6,height=4.5)



####Now distance to centroid
test2 <- t.test(DistCent~Status, data=xbin2)

#p=0.4 distributions aren't different

###Plot distance to nearest neighbor
ggplot(data=xbin2,aes(x=Status,y=DistCent,fill=Status))+
geom_boxplot(lwd=2)+

scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test2$p.value,digits=2),sep=""))+

#Formatting
scale_y_continuous(name="Functional distance to centroid")+
#ggtitle("Distance to centroid")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

ggsave(filename="Figure S14.pdf",width=6,height=4.5)


#########
#Looking at whether pairs that were short correlate with extinction

#Pull out pairs that are listed twice
xbin3 <- xbin2[duplicated(xbin2$Dist1Pair)==F,]
dim(xbin3)#64 25

#One way to look at it
ggplot(data=xbin3, aes(x=Dist1,fill=PairExtinct))+
geom_density(alpha=.5)


#Make a nice plot
#First need to reorder the factors
xbin3$PairExtinct <- with(xbin3,factor(PairExtinct, levels=c(c("None","One","Both"))))


test3 <- kruskal.test(Dist1~as.factor(PairExtinct), data=xbin3)#p .14

#Data points are not independant because they are pairs of species. A species could occur multiple times in the same group or in other groups.


ggplot(data=xbin3, aes(x=PairExtinct,y=Dist1,fill=PairExtinct))+
geom_boxplot(size=2)+

scale_fill_manual(name="PairExtinct",values=c("None"="lightskyblue","One"="#ff7777","Both"="firebrick3"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test3$p.value,digits=2),sep=""))+

#Formatting
scale_y_continuous(name="Functional distance to nearest neighbor")+
scale_x_discrete(name="Number of species in a pair that go extinct")+
#ggtitle("Distance does not determine maintenance of closest species pairs")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

ggsave(filename="Figure S15.pdf",width=6,height=4.5)


rm(list = ls())
#########################



#Look at functional distances over time

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Set up the weights.  Outside the function call.
#Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))


#Run the function that calculates the unique pendant edge for each species in each bin
#This takes a few seconds
source(file.path(mainDir,"Davis Unique Branchlengths.R"), chdir = F)
dim(allbin)#3808 39
head(allbin)
tail(allbin)

#Do some plotting
require(ggplot2)
require(plyr)
require(reshape2)
require(Hmisc)


#Trim the data down a bit
colnames(allbin)
allbin1 <- allbin[,c(1:6,34:38)]
head(allbin1)
colnames(allbin1)


#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)


allbin2 <- allbin1

allbin3 <- allbin2[allbin2$Status!="Introduced" &allbin2$Bin!=max(unique(allbin2$Bin)),]
dim(allbin3)#3753 11
tail(allbin3)


#And calculate means up top
colnames(allbin3)
binmeans <- ddply(allbin3, .(Status,Bin),summarise, Age=mean(Age), RelUniq=mean(RelUniq), RelFD=mean(RelFD))
binmeans

allbin3gone <- subset(allbin3,Status=="Extinct")
allbin3here <- subset(allbin3,Status=="Extant")


###First plot survivors
ggplot(data=allbin3here,aes(x=Age,y=100*RelUniq,color=Status, group=Binomial))+

#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extant"),aes(x=Age, y=100*RelUniq,group=Status),colour="dodgerblue2",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_y_continuous(name="Dendrogram functional uniqueness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Functional uniqueness for each species over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelUniq)-0.5,max(100*allbin3$RelUniq)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")
#theme(legend.position="bottom")
#Survivor that catapults to the top is a giant anteater
#allbin3[allbin3$Bin==1,]
#Survivor in the last future bin that jumps up is brown bear

ggsave(filename="Figure S8.pdf",width=6,height=4.5)




###Now extinct species
ggplot(data=allbin3gone,aes(x=Age,y=100*RelUniq,color=Status, group=Binomial))+


#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extinct"),aes(x=Age, y=100*RelUniq,group=Status),colour="red1",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_y_continuous(name="Dendrogram functional uniqueness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Functional uniqueness for each species over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelUniq)-0.5,max(100*allbin3$RelUniq)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")

ggsave(filename="Figure S9.pdf",width=6,height=4.5)



#A methodological note
allbin3gone[allbin3gone$RelUniq==max(allbin3gone$RelUniq),]
#When the wooly mammoth goes extinct in bin 13, it takes it's whole clade with it but also it takes the branch leading from the root to the other basal clade.  This means, it's RelUniq is actually larger than it's RelFD because that extra bit of branch is not included in the RelFD calculation for the mammoth.


#############

#Now plot the same for dendrogram functional distinctiveness. I want to show species over time how their proportional FD rises

#Means were already calculatedup top

####First survivors
ggplot(data=allbin3here,aes(x=Age,y=100*RelFD,color=Status, group=Binomial))+

#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extant"),aes(x=Age, y=100*RelFD,group=Status),colour="dodgerblue2",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_y_continuous(name="Functional distinctiveness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Tree based proportional FD over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelUniq)-0.5,max(100*allbin3$RelUniq)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")
#Survivor that catapults to the top is a giant anteater
#allbin3[allbin3$Bin==1,]
#Survivor in the last future bin that jumps up in brown bear
ggsave(filename="Figure S5.pdf",width=6,height=4.5)




###Now extinct species
ggplot(data=allbin3gone,aes(x=Age,y=100*RelFD,color=Status, group=Binomial))+


#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extinct"),aes(x=Age, y=100*RelFD,group=Status),colour="red1",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_y_continuous(name="Functional distinctiveness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Tree based proportional FD over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelUniq)-0.5,max(100*allbin3$RelUniq)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")


ggsave(filename="Figure S6.pdf",width=6,height=4.5)


rm(list = ls())
#########################


#Now look at pca space measures of relative uniqueness

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)

#Trim to just native species
allrange <- allrange[allrange$FAD>49000,]
dim(allrange)# 85 33

#And quickly get their pca traits
traits <- allrange[,c(9,11:19)]
#Set up the weights
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))
traits <- as.matrix(traits)
#Traits Z transformed so they have a sd of 1 and mean of 0
ndf <- apply(traits, 2, scale, center = T, scale = T)
#Now apply weighting to the traits
ndf2 <- t(t(ndf)*weights)
traits2 <- ndf2
rownames(traits2) <- rownames(allrange)
#Do the pca
pca1 <- prcomp(traits2,center=F,scale=F)
pctraits <- pca1$x[,1:4]

results <- list()
results$pctraits <- pctraits

#Now run this function which will figure out how much FRic decreases when a species is lost
source(file.path(mainDir,"Davis Convex Hull.R"), chdir = F)

colnames(allbin)
allbin[1:200,35:37]
allbin[200:300,35:37]

#Just remove the earliest bin
xbin <- allbin[allbin$Bin==52,]
dim(xbin)#85 37



#Let's do some plotting
#require(reshape2)
#require(plyr)
require(ggplot2)
#require(grid)
#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)



test1 <- wilcox.test(RelFRic~Status, data=xbin)

#p=0.51 distributions aren't different


ggplot(data=xbin,aes(x=Status,y=100*RelFRic,fill=Status))+
geom_boxplot(size=2)+
scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#annotate("text",x=Inf,y=Inf,vjust=2,hjust=1.25,size=8,label=paste("p=",round(test1$p.value,digits=2),sep=""))+

#Formatting
scale_y_continuous(name="PCA space functional uniqueness")+
#One that is zoomed in
#coord_cartesian(ylim=c(0,2))+

#ggtitle("Extinct species occur throughout functional space")+
simpleblack+
#presentmod+
#To put the legend on the bottom
theme(legend.position="none")

ggsave(filename="Figure S10.pdf",width=6,height=4.5)



rm(list = ls())
#########################


#Now look at pca space measures of relative uniqueness over time

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


#Also load in functional results to get correct number of axes to keep and weights
load(file.path(mainDir,"Function Results"))
str(results)

#Load this function which will show how much of FRic is lost if a species goes extinct during that timebin
source(file.path(mainDir,"Davis Convex Hull.R"), chdir = F)

colnames(allbin)
allbin[1:200,35:37]
allbin[200:300,35:37]

#Let's do some plotting
require(reshape2)
require(plyr)
require(ggplot2)
#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)


#Trim the data down a bit
colnames(allbin)
allbin1 <- allbin[,c(1:6,34:37)]
head(allbin1)
colnames(allbin1)



allbin2 <- allbin1

allbin3 <- allbin2[allbin2$Status!="Introduced" &allbin2$Bin!=max(unique(allbin2$Bin)),]
dim(allbin3)#3753 10



#And calculate means up top
colnames(allbin3)
binmeans <- ddply(allbin3, .(Status,Bin),summarise, Age=mean(Age), RelFRic=mean(RelFRic))
binmeans

allbin3gone <- subset(allbin3,Status=="Extinct")
allbin3here <- subset(allbin3,Status=="Extant")

####For extant
#ggplot(data=allbin3,aes(x=Age,y=100*RelFRic,color=Status, group=Binomial))+
ggplot(data=allbin3here,aes(x=Age,y=100*RelFRic,color=Status, group=Binomial))+

#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extant"),aes(x=Age, y=100*RelFRic,group=Status),colour="dodgerblue2",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#Formatting
scale_y_continuous(name="PCA space functional uniqueness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Proportional FRic for each species over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelFRic)-0.5,max(100*allbin3$RelFRic)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")

ggsave(filename="Figure S11.pdf",width=6,height=4.5)



####Now extinct
ggplot(data=allbin3gone,aes(x=Age,y=100*RelFRic,color=Status, group=Binomial))+

#geom_text(aes(label=Binomial),size=1.5)+
geom_path(size=0.75,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot up each mean seperately
geom_path(data=subset(binmeans,Status=="Extinct"),aes(x=Age, y=100*RelFRic,group=Status),colour="red1",size=3,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+

#Formatting
scale_y_continuous(name="PCA space functional uniqueness")+
scale_x_reverse(name="Years before present")+
#ggtitle("Proportional FRic for each species over time\nThick lines are means")+
coord_cartesian(ylim=c(min(100*allbin3$RelFRic)-0.5,max(100*allbin3$RelFRic)+1))+
simpleblack+
#presentmod+
theme(legend.position="none")

ggsave(filename="Figure S12.pdf",width=6,height=4.5)






allbin[allbin$Bin==1,]

allbin[allbin$Bin==3,]
#Extinct that is around 7 percent then jumps up is Columbian mammoth
#Extant lontra canadensis is the highest pop up at about 23% after Lontra longicaudus is at 1% when it goes extinct
#Rangifer is pretty constant at around 10%
#Panthera onca is responsible for 12% of FRic when it goes extint in the future
#Giant anteater is responsible for 35% when it goes extinct
#Polar bear only responsible for 6%, Tapirus bairdii only 1%
#Bison goes from 15% pre columbus to 4% 


rm(list = ls())
###########################



#Now compare distances to centroid and nearest neighbor over time


#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


#Also load in functional results to get correct number of axes to keep and weights
load(file.path(mainDir,"Function Results"))
str(results)
m <- results[["numaxes"]]
weights <- results[["weights"]]

#Run the nearestneighbor1.4 function
source(file.path(mainDir,"Davis Nearest Neighbor.R"), chdir = F)
#Takes a few seconds

#Remove last bin because it is fake
allbin <- allbin[allbin$Bin!=53,]
dim(allbin)#3788 25
head(allbin)
tail(allbin)


#What do the two look like togeher?
#Drop introduced
allbin3 <- allbin[allbin$Status!="Introduced",]
allbin3$Status <- droplevels(allbin3$Status)

#And calculate means up top
colnames(allbin3)
binmeans <- ddply(allbin3, .(Status,Bin),summarise, Dist1=mean(Dist1),DistCent=mean(DistCent))

####
require(grid)
require(ggplot2)
#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)

#Now plot it up
ggplot(data=allbin3,aes(x=DistCent,y=Dist1,color=Status))+
#Plot all the individual lines
geom_path(aes(group=Binomial),size=0.8,arrow=arrow(angle=15, ends="first", type="closed",length=unit(2, "mm")))+

#Now plot the means separately 
geom_path(data=subset(binmeans,Status=="Extant"),aes(x=DistCent, y=Dist1),color="dodgerblue2",size=2,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

geom_path(data=subset(binmeans,Status=="Extinct"),aes(x=DistCent, y=Dist1),color="red1",size=2,arrow=arrow(angle=15, ends="first", type="closed",length=unit(4, "mm")))+

scale_color_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+


#Formatting
scale_y_continuous(name="Functional distance to nearest neighbor",limits=c(0,2/4*max(allbin$DistCent)))+
scale_x_continuous(name="Functional distance to centroid")+
coord_fixed()+
#ggtitle("Functional distances for each species over time\nThick lines are means")+
simpleblack+
#presentmod+
theme(legend.position="none",legend.title=element_blank())


#ggsave(filename="Figure S14 Supplimental.pdf",width=6,height=4.5)

#Checking
#allbin3[allbin3$Binomial=="Mammuthus primigenius",]
#Looks good arrows in right place

rm(list = ls())

###########################################################


###
#COMPARISON TO VALUES DERIVED FROM GOWER'S DISTANCE
###


#FUNCTIONAL SPACE

#Show what the species functional tree and space look like 
require(picante)
require(Hmisc)	
require(cluster)
require(FD)

#First get the working directory
mainDir <- getwd()

#Load up the traits
allrange <- read.csv(file=file.path(mainDir,"Allrange.csv"), row.names=1)
dim(allrange)#94 33
head(allrange)


#Pull out traits, caculate distance and create the tree

#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9,11:19)]
traits



#Set up the weights.  Weighting so mass, diet, and locomotion/sociality all get a third
groupweight <- 1/3
weights <- c(groupweight,rep(groupweight/4,4),rep(groupweight/5,5))


#Change the variable type from continuous to ordered for the locomotor traits
colnames(traits)
gtraits <- traits
gtraits[,6] <- as.ordered(traits[,6])
gtraits[,7] <- as.ordered(traits[,7])
gtraits[,8] <- as.ordered(traits[,8])
gtraits[,9] <- as.ordered(traits[,9])	
gtraits
gtraits$Swimming

#Now get gowers distance
gdist1 <- gowdis(gtraits, weights, ord="podani")


#####
#Now get pseudo Euclidean distance
traits <- as.matrix(traits)

#Traits Z transformed so they have a sd of 1 and mean of 0
ndf <- apply(traits, 2, scale, center = T, scale = T)

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traitsEuclid <- ndf2
rownames(traitsEuclid) <- rownames(allrange)

#Make the distance matrix, don't want standardization because we already standardized and weighted
edist1 <- daisy(traitsEuclid, metric="euclidean",stand=F)

#Now plot
require(ggplot2)

#To automatically load in plot parameters
source(file.path(mainDir,"Davis Plot Parameters.R"), chdir = F)




#Turn the distances into something ggplot understands
vedist1 <- c(edist1)
vgdist1 <- c(gdist1)

distdf <- as.data.frame(cbind(vedist1,vgdist1))
colnames(distdf) <- c("Euclidean","Gowers")

#Points aren't independent but can put in a regression to guide the eye 
ggplot(data=distdf,aes(x=Euclidean,y=Gowers))+
geom_point(alpha=0.25)+
#geom_smooth(method="lm",size=3,color="grey")+
scale_x_continuous(name="Pseudo Euclidean distance")+
scale_y_continuous(name="Gower's distance")+
simpleblack
ggsave(filename="Figure S22.pdf",width=6,height=4.5)

#############################

#Now plot Functional dendrogram made from gowers distance

#Now make the functional dendrogram
tree <- hclust(gdist1,method="average")
plot(tree)


#Check the correlation between tree distance and euclidean distance ~a measure of tree quality to capture all the data
treedist <- cophenetic(tree)
copheneticcorrelation <- cor(gdist1, treedist)#.74


ftree <- as.phylo(tree)
str(ftree)

require(ggplot2)
require(ggtree)
require(gridExtra)

fulltree <- ftree
#These are the tips to drop
prunetips <- allrange[allrange$Status=="Extinct","Labels"]

#But get the tips to keep
keeptips <- fulltree$tip.label[!fulltree$tip.label %in% prunetips]

#Group the tips to keep
prunetree <- groupOTU(fulltree, focus=keeptips)

#And plot
#First column of data has to match tiplabels of tree
colnames(allrange)
allrange2 <- cbind(allrange[,"Labels"],allrange)
colnames(allrange2)[1] <- "Taxa"
colnames(allrange2)

#Create base tree
p <- ggtree(prunetree,layout="circular", aes(color=group))
#Sync data
p <- p %<+% allrange2 

#Plot
p2 <- p+
scale_color_manual(values=c("lightgrey","black","dodgerblue2","red1","green4"))+
geom_tiplab(aes(subset=(abs(angle)<90),angle=angle,col=Status,label=Binomial),fontface="italic",size=3)+
geom_tiplab(aes(subset=(abs(angle)>=90),angle=angle+180,col=Status,label=Binomial), fontface="italic",size=3,hjust=1)+
xlim(0,0.25)
p2


ggsave(filename="Figure S23.pdf",width=8,height=8)

###########################


#Plot PCO Trait space
require(labdsv)
#Do the pco
pco1 <- pco(gdist1,k=dim(gtraits)[2])
pco1


origscore <- pco1$points
toteig <- sum(pco1$eig[pco1$eig>0])

#And Figure out the proportion of variance
eig1 <- round((pco1$eig[1]/toteig)*100,digits=0)
eig2 <- round((pco1$eig[2]/toteig)*100,digits=0)
eig3 <- round((pco1$eig[3]/toteig)*100,digits=0)
eig4 <- round((pco1$eig[4]/toteig)*100,digits=0)
#Explains 42, 17, 10, 7 % of variance
eig1+eig2+eig3+eig4 #Total 76%


#Checking loadings
require(vegan)
ef <- envfit(origscore, gtraits, permu=999)
ef

#Plot version with loadings
#pdf(width=10,height=7.5, file="Figure S21 Gowers.pdf")
plot(origscore[,1:2], type="n",xlab=paste("PC1 (",eig1,"% of variance)",sep=""),ylab=paste("PC2 (",eig2,"% of variance)",sep=""))
text(origscore[,1:2], rownames(origscore), cex=0.3)
plot(ef, col="red")
#dev.off()



#Get native and extant species
natives <- origscore[allrange$FAD>49000,1:2]
modern <- origscore[allrange$Status=="Extant",1:2]

#Get the convex hull values
nativehull <- chull(natives)
nativehull <- c(nativehull,nativehull[1])

modernhull <- chull(modern)
modernhull <- c(modernhull,modernhull[1])


#Plot PCA Space with convex hulls
pdf(width=6,height=6, file="Figure S24.pdf")

#Reverse the x axis so it matches the previous pca
plot(origscore[,1:2],type="n",xlab=paste("PCO1 (",eig1,"% of variance)",sep=""),ylab=paste("PCO2 (",eig2,"% of variance)",sep=""),xlim=rev(range(origscore[,1])))

#Plot convex hulls
lines(natives[nativehull,1:2],col="#ff7777",lwd=3.4)
lines(modern[modernhull,1:2],col="lightskyblue",lwd=4.2)

#Plot points
points(origscore[allrange$Status=="Extinct",1:2],col="red1",pch=19)
points(origscore[allrange$Status=="Extant",1:2],col="dodgerblue2",pch=19)
points(origscore[allrange$Status=="Introduced",1:2],col="green4",pch=19)

dev.off()

#And a plot for the inset
#pdf(width=6,height=6, file="Figure 24 inset.pdf")

#plot(origscore[,1:2],type="n")
#plot(ef, col="black")
#dev.off()
##########################


#Now calculate functional diveristy over time

#Need to set new times for possible future extinctions
#By default, endangered species will get a LAD of 0
#Give a LAD of -500 (500 years in the future) to those species that are near threatened
allrange[allrange$Red.List.status=="NT",32] <- -500

#And give a LAD of -1500 to those species that aren't threatened
allrange[allrange$Red.List.status=="DD"| allrange$Red.List.status=="LC",32] <- -1500



#Create species timebin occurence matrix
alltaxa <- rownames(traits)
numtaxa <- dim(traits)[1]
numtraits <- dim(traits)[2]
dates <- allrange[,31:32]
	#For equal time bins including possible future extinctions
	inoutdates1 <-seq(-2000,50000,by=1000)
	
	timesteps <- length(inoutdates1)
	
	bins <- inoutdates1
	
	#Make the occurence matrix
	#Need to add one row so that dbFD reads a step with all species and species iterations in the matrix

	therematrix <- matrix(data=0, nrow=timesteps, ncol=numtaxa, dimnames=list(c(inoutdates1),rownames(dates)))
	#dim(therematrix)#53 94


	#Now fill it up
	#For each species
	for(i in 1:numtaxa){
	
	thistaxa <- alltaxa[i]
	
	#Take out lads and fads
	FAD <- dates[i,1]
	LAD <- dates[i,2]
	
		#For each timestep
		for(j in 1:timesteps){
	
			#And fill up only that taxa's column 
			therematrix[j,thistaxa] <- 	ifelse((FAD>=bins[j] & LAD<bins[j+1]),1,0)


			
		}#End j loop
	}#End i loop

	#Insert all 1's in the last row so that there a fake timeslice with all species 
	
	therematrix[timesteps,] <- 1
	


###

#Calculate Functional Diversity


	#Make a matrix of the results
	 data1<- as.data.frame(matrix(data=NA, nrow=timesteps, ncol=7, dimnames=list(rownames(therematrix),c("SpeciesRichness", "FD", "FRic","FDis","RaoQ","Bin","Date"))))

	data1$Date <- bins
	data1$Bin <- c(1:timesteps)
	
###

#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure
	FD1 <- pd(therematrix, ftree, include.root=F)
	
	#Put it in the results matrix
	data1$SpeciesRichness <- FD1$SR
	
	#Standardize FD
	data1$FD <- FD1$PD/max(FD1$PD)
	
	
###

#Now for FRic, RaoQ and FDis
#Takes a while
gfd <- dbFD(x=gtraits,a=therematrix,w=weights,m=numtraits,ord="podani",corr="cailliez",stand.FRic=T)

#Quality of reduced space was 0.40

gfd

colnames(data1)

data1$FRic <- gfd$FRic
data1$FDis <- gfd$FDis
data1$RaoQ <- gfd$RaoQ

gdata1 <- data1

save(gdata1, file="Function Results Gowers")

rm(list = ls())
###########################


#Now load up gowers and pseudo Euclidean results and compare

#Load in pseudo Euclidean results of functional diversity analysis
load("Function Results")
str(results)
edata1 <- results$data1

#Now load in gowers distance results of functional diversity analysis
load("Function Results Gowers")
gdata1

require(ggplot2)
require(gridExtra)
require(plyr)
require(reshape2)

#Get plotting themes
source('Davis Plot Parameters.R', chdir = F)


#Add relative richness category 
edata1$RelRich <- edata1$SpeciesRichness/max(edata1$SpeciesRichness)

#Delete last row because it is the fake bin
tail(edata1)
edata2 <- edata1[-dim(edata1)[1],]

#Change FD and FRic so they are relative to starting position
edata2$FD <- edata2$FD/edata2$FD[length(edata2$FD)]
edata2$FRic <- edata2$FRic/edata2$FRic[length(edata2$FRic)]


#Change FDis and RaoQ so they are relative to starting position
edata2$FDis <- edata2$FDis/edata2$FDis[length(edata2$FDis)]
edata2$RaoQ <- edata2$RaoQ/edata2$RaoQ[length(edata2$RaoQ)]


#Melt the pseudo Euclidean data
melte <- melt(edata2,id.vars=c("RelRich","Bin","Date"))
head(melte)
colnames(melte)[4:5] <- c("Index","Value")
#Add column for distance metric
melte$Distance <- "Pseudo Euclidean"
head(melte)
tail(melte)


#Now for gowers distance

#Add relative richness category 
gdata1$RelRich <- gdata1$SpeciesRichness/max(gdata1$SpeciesRichness)

#Delete last row because it is the fake bin
tail(gdata1)
gdata2 <- gdata1[-dim(gdata1)[1],]

#Change FD and FRic so they are relative to starting position
gdata2$FD <- gdata2$FD/gdata2$FD[length(gdata2$FD)]
gdata2$FRic <- gdata2$FRic/gdata2$FRic[length(gdata2$FRic)]


#Change FDis and RaoQ so they are relative to starting position
gdata2$FDis <- gdata2$FDis/gdata2$FDis[length(gdata2$FDis)]
gdata2$RaoQ <- gdata2$RaoQ/gdata2$RaoQ[length(gdata2$RaoQ)]

#Melt the pseudo Euclidean data
meltg <- melt(gdata2,id.vars=c("RelRich","Bin","Date"))
head(meltg)
colnames(meltg)[4:5] <- c("Index","Value")
#Add column for distance metric
meltg$Distance <- "Gower's"
head(meltg)
tail(meltg)

#Join them together
meltdata <- rbind(melte,meltg)

gradient<-colorRampPalette(c("#00008B","#68228B","#FF1493","#FF8C00","#FFE4C4"))

#Plot distances against each other first
###FRic
plot1 <- ggplot(data=edata2, aes(x=FRic,y=gdata2$FRic,color=Date))+
geom_abline(slope=1,intercept=c(0,0),linetype="dashed",color="grey",size=1.5)+
geom_line(size=1.5)+
geom_point(size=3)+
#annotate("text",x=Inf,y=Inf,vjust=2,hjust=7,size=8,label="(a)")+
coord_fixed()+
scale_color_gradientn(colors=gradient(dim(edata2)[1]))+
scale_y_continuous(limits=c(0,1),name="FRic (Relative to starting value):\nGower's distance")+
scale_x_continuous(limits=c(0,1),name="FRic (Relative to starting value):\npseudo Euclidean distance")+
theme(legend.position="none")+
ggtitle("(a)")+
simpleblack


###FD
plot2 <- ggplot(data=edata2, aes(x=FD,y=gdata2$FD,color=Date))+
geom_abline(slope=1,intercept=c(0,0),linetype="dashed",color="grey",size=1.5)+
geom_line(size=1.5)+
geom_point(size=3)+
#annotate("text",x=Inf,y=Inf,vjust=2,hjust=7,size=8,label="(b)")+
coord_fixed()+
scale_color_gradientn(colors=gradient(dim(edata2)[1]))+
scale_y_continuous(limits=range(c(gdata2$FD,edata2$FD)),name="FD (Relative to starting value):\nGower's distance")+
scale_x_continuous(limits=range(c(gdata2$FD,edata2$FD)),name="FD (Relative to starting value):\npseudo Euclidean distance")+
theme(legend.position="none")+
ggtitle("(b)")+
simpleblack


###FDis
plot3 <- ggplot(data=edata2, aes(x=FDis,y=gdata2$FDis,color=Date))+
geom_abline(slope=1,intercept=c(0,0),linetype="dashed",color="grey",size=1.5)+
geom_line(size=1.5)+
geom_point(size=3)+
#annotate("text",x=Inf,y=Inf,vjust=2,hjust=7,size=8,label="(c)")+
coord_fixed()+
scale_color_gradientn(colors=gradient(dim(edata2)[1]))+
scale_y_continuous(limits=range(c(gdata2$FDis,edata2$FDis)),name="FDis (Relative to starting value):\nGower's distance")+
scale_x_continuous(limits=range(c(gdata2$FDis,edata2$FDis)),name="FDis (Relative to starting value):\npseudo Euclidean distance")+
theme(legend.position="none")+
ggtitle("(c)")+
simpleblack

###RaoQ
plot4 <- ggplot(data=edata2, aes(x=RaoQ,y=gdata2$RaoQ,color=Date))+
geom_abline(slope=1,intercept=c(0,0),linetype="dashed",color="grey",size=1.5)+
geom_line(size=1.5)+
geom_point(size=3)+
#annotate("text",x=Inf,y=Inf,vjust=2,hjust=7,size=8,label="(d)")+
coord_fixed()+
scale_color_gradientn(colors=gradient(dim(edata2)[1]))+
scale_y_continuous(limits=range(c(gdata2$RaoQ,edata2$RaoQ)),name="RaoQ (Relative to starting value):\nGower's distance")+
scale_x_continuous(limits=range(c(gdata2$RaoQ,edata2$RaoQ)),name="RaoQ (Relative to starting value):\npseudo Euclidean distance")+
theme(legend.justification=c(1,0), legend.position=c(1,0))+
ggtitle("(d)")+
simpleblack

grid.arrange(plot1,plot2,plot3,plot4,ncol=2)
g <- arrangeGrob(plot1,plot2,plot3,plot4,ncol=2)

ggsave(g,filename="Figure S25.pdf",width=8,height=6)


rm(list = ls())
###########################

#Now look at values over time
#Need to start over to get FD and FRic data that isn't standardized to start values

#Now load up gowers and pseudo Euclidean results and compare

#Load in pseudo Euclidean results of functional diversity analysis
load("Function Results")
str(results)
edata1 <- results$data1

#Now load in gowers distance results of functional diversity analysis
load("Function Results Gowers")
gdata1

require(ggplot2)
require(gridExtra)
require(plyr)
require(reshape2)

#Get plotting themes
source('Davis Plot Parameters.R', chdir = F)


#Add relative richness category 
edata1$RelRich <- edata1$SpeciesRichness/max(edata1$SpeciesRichness)

#Delete last row because it is the fake bin
tail(edata1)
edata2 <- edata1[-dim(edata1)[1],]


#Change FDis and RaoQ so they are relative to starting position
edata2$FDis <- edata2$FDis/edata2$FDis[length(edata2$FDis)]
edata2$RaoQ <- edata2$RaoQ/edata2$RaoQ[length(edata2$RaoQ)]


#Melt the pseudo Euclidean data
melte <- melt(edata2,id.vars=c("RelRich","Bin","Date"))
head(melte)
colnames(melte)[4:5] <- c("Index","Value")
#Add column for distance metric
melte$Distance <- "Pseudo Euclidean"
head(melte)
tail(melte)


#Now for gowers distance

#Add relative richness category 
gdata1$RelRich <- gdata1$SpeciesRichness/max(gdata1$SpeciesRichness)

#Delete last row because it is the fake bin
tail(gdata1)
gdata2 <- gdata1[-dim(gdata1)[1],]


#Change FDis and RaoQ so they are relative to starting position
gdata2$FDis <- gdata2$FDis/gdata2$FDis[length(gdata2$FDis)]
gdata2$RaoQ <- gdata2$RaoQ/gdata2$RaoQ[length(gdata2$RaoQ)]

#Melt the pseudo Euclidean data
meltg <- melt(gdata2,id.vars=c("RelRich","Bin","Date"))
head(meltg)
colnames(meltg)[4:5] <- c("Index","Value")
#Add column for distance metric
meltg$Distance <- "Gower's"
head(meltg)
tail(meltg)

#Join them together
meltdata <- rbind(melte,meltg)



#Subset data to just functional diversity indices
funcmelt <- subset(meltdata,Index %in% c("FD","FRic","FDis","RaoQ"))





ggplot(data=funcmelt,aes(x=Date,y=Value,group=c(Distance),color=Distance))+
geom_line(size=1)+

#Set up colors
scale_color_manual(name="Index",values=c("mediumaquamarine","black"))+

facet_grid(Index ~., scales="free_y")+
scale_x_reverse(name="Years before present")+
scale_y_continuous(name=" \n ")+
theme(legend.position="none")+
#Remove labels
theme(strip.background=element_blank(),strip.text=element_blank())+

#labs(title="Functional index values over time\nColored lines represent median and 95% CI from 1000 simulation runs")+
simpleblack



ggsave(filename="Figure S26.pdf",width=6,height=4.5)

#####################################################
#####################################################



###
#TOTAL MAMMAL FUNCTIONAL SPACE SIMULATIONS
###


#CREATE TOTAL MAMMAL FUNCTIONAL DATA SET

#This will test how including smaller species could change results.
#It was adapted on November 2, 2016 from Quaternary Function Basefile 3.5.R in the the author's own version control system



#First load up the original functional dataset for large extant and extinct mammals
allrange <- read.csv(file="Allrange.csv", row.names=1)
dim(allrange)#94 33
head(allrange)
#Trim to just data that we will need
colnames(allrange)
allrange1 <- allrange[,c(1:9,11:14,30)]
colnames(allrange1)
dim(allrange1)#94 14

#Now load up data for the 8 small species of mammal that are thought to have gone extinct during the last 50,000 years
small <- read.csv(file="Small Extinct Mammal Functional Data.csv", comment.char="#")
dim(small)#8 49
small
#Trim out uneeded columns and match up with allrange
colnames(small)
small2 <- small[,c(1:9,11:14)]
small2$Labels <- paste(small2$Genus, "_", small2$Species, sep="")
rownames(small2) <- small2$Labels
dim(small2)#8 14

#Join the small mammals to the large mammals

allrange2 <- rbind(allrange1, small2)
allrange2



#Now load up a functional dataset for extinct mammals. This is functional data from EltonTraits1.0 trimmed to only North American terrestrial mammals using the Mass of Mammals database 

#Wilman, H., J. Belmaker, J. Simpson, C. de la Rosa, M. M. Rivadeneira, and W. Jetz. 2014. EltonTraits 1.0: species-level foraging attributes of the world’s birds and mammals. Ecology 95:2027.

#Smith, F. A., S. K. Lyons, S. Ernest, K. Jones, D. Kaufman, T. Dayan, P. Marquet, et al. 2003. Body mass of late quaternary mammals. Ecology 84:3403. Version 4.1

elton <- read.csv(file="EltonTraits1.0 North American Mammals.csv")
dim(elton)#686 30
head(elton)
#Make a log mass in g column
elton$Log10Massg <- log10(elton$BodyMass.Value)
#And labels
elton$Labels <- paste(elton$Genus, "_", elton$Species, sep="")
rownames(elton) <- elton$Labels

#Now remove all the large mammals already covered by allrange
#Min is 3.592
elton2 <- elton[elton$Log10Massg<min(allrange$Log10Massg),]
dim(elton2)#634 32
head(elton2)
colnames(elton2)


#Now make an empty data frame that this modified functional data will go into to get it into the allrange format
elrange <- as.data.frame(matrix(data=NA, ncol=length(colnames(allrange2)), nrow=dim(elton2)[1]))
colnames(elrange) <- colnames(allrange2)
rownames(elrange) <- rownames(elton2)
head(elrange)
colnames(elrange)

#And fill it up
#Easy things first
elrange[,1:6] <- elton2[,1:6]
elrange$Labels <- elton2$Labels
elrange$Log10Massg <- elton2$Log10Massg

#Now diet
elrange$Invertebrates <- elton2$Diet.Inv

elrange$Vertebrates <- elton2$Diet.Vend+elton2$Diet.Vect+elton2$Diet.Vfish+elton2$Diet.Vunk+elton2$Diet.Scav

#Browse and Graze are hard because we don't know whether seeds and plant other are Browse or Graze so we are agnostically putting half in browse and half in graze
elrange$Browse <- elton2$Diet.Fruit+elton2$Diet.Nect+ ((elton2$Diet.Seed+elton2$Diet.PlantO)/2)

elrange$Graze <- ((elton2$Diet.Seed+elton2$Diet.PlantO)/2)

#Check
head(elrange)
any(rowSums(elrange[,10:13])!=100)


#Now join those to the big and small mammals
allrange3 <- rbind(allrange2, elrange)
dim(allrange3)#736 14
allrange3$Binomial <- as.character(allrange3$Binomial)

allrange4 <- allrange3[order(allrange3$Binomial),]
head(allrange4)
tail(allrange4)

#Check for duplicates and remove them
allrange4$Binomial[duplicated(allrange4$Binomial)]

allrange4[allrange4$Binomial=="Leopardus wiedii",]
#This species was duplicated because of the Margay's complicated taxonomy. Keep the row from the original allrange dataset
allrange5 <- allrange4[!rownames(allrange4)=="Leopardus_wiedii1",]
dim(allrange5)#735 14

summary(allrange5)

#Write it
write.csv(allrange5, file="Allrange All North American.csv")

rm(list = ls())
##################################################################################################################

#PLOT TOTAL MAMMAL FUNCTIONAL SPACE

require(picante)
require(Hmisc)	
require(cluster)

#Plot out functional space for all North American mammals
allrange3 <- read.csv(file="Allrange All North American.csv", row.names=1)
dim(allrange3)#735 14
head(allrange3)
tail(allrange3)

#Take out all introduced species
allrange2 <- allrange3[allrange3$Status!="Introduced",]
dim(allrange2)#728 14

#And take out Homo sapiens and Bison bison because they arrived after 50,000 years.
allrange <- allrange2[allrange2$Binomial!="Homo sapiens" & allrange2$Binomial!="Bison bison" ,]
dim(allrange)#726 14

summary(allrange)
#That leaves us with 676 Extant species and 50 Extinct species with mean log10 mass of 2.0850

#Now let's get an idea of what functional space looks like


#Pull out traits, caculate distance and create the tree

#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9:13)]
traits

#Set up the weights.  Weighting so mass and diet each get half 
groupweight <- 1/2
weights <- c(groupweight,rep(groupweight/4,4))


	
#Bookkeeping
traits <- as.matrix(traits)

#Traits Z transformed so they have a sd of 1 and mean of 0
ndf <- apply(traits, 2, scale, center = T, scale = T)

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2
rownames(traits2) <- rownames(allrange)

#############################

#Plot Functional dendrogram

#Make the distance matrix, don't want standardization because we already standardized and weighted
dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
#Now make the functional dendrogram
tree <- hclust(dist1,method="average")
plot(tree)


#Check the correlation between tree distance and Euclidean distance ~a measure of tree quality to capture all the data
treedist <- cophenetic(tree)
copheneticcorrelation <- cor(dist1, treedist)#.82

#To make a better version of that tree
ftree <- as.phylo(tree)
str(ftree)

require(ggplot2)
require(ggtree)
require(gridExtra)



fulltree <- ftree
#These are the tips to drop
prunetips <- allrange[allrange$Status=="Extinct","Labels"]

#But get the tips to keep
keeptips <- fulltree$tip.label[!fulltree$tip.label %in% prunetips]

#Group the tips to keep
prunetree <- groupOTU(fulltree, focus=keeptips)

#And plot
#First column of data has to match tiplabels of tree
colnames(allrange)
allrange2 <- cbind(allrange[,"Labels"],allrange)
colnames(allrange2)[1] <- "Taxa"
colnames(allrange2)

#Create base tree
p <- ggtree(prunetree,layout="circular", aes(color=group), size=0.3)
#Sync data
p <- p %<+% allrange2 

#Plot
p2 <- p+
scale_color_manual(values=c("lightgrey","black","dodgerblue2","red1","green4"))+
geom_tiplab(aes(subset=(abs(angle)<90),angle=angle,col=Status,label=Binomial), fontface="italic",size=0.4)+
geom_tiplab(aes(subset=(abs(angle)>=90),angle=angle+180,col=Status,label=Binomial), fontface="italic",size=0.4,hjust=1)+
xlim(0,0.65)
p2


ggsave(filename="Figure S27.pdf",width=6, height=4.5)

###########################


#Plot PCA Trait space

#Do the pca, traits were already centered and scaled
pca1 <- prcomp(traits2,center=F,scale=F)


origscore <- pca1$x


#And Figure out the proportion of variance
eig1 <- round((summary(pca1)$importance[2,1])*100,digits=0)
eig2 <- round((summary(pca1)$importance[2,2])*100,digits=0)
eig3 <- round((summary(pca1)$importance[2,3])*100,digits=0)
eig4 <- round((summary(pca1)$importance[2,4])*100,digits=0)
#Explains 82, 10, 4, 3 % of variance
eig1+eig2+eig3+eig4 #Total 99%


#Checking loadings
require(vegan)
ef <- envfit(origscore, traits2, permu=999)
ef

str(ef)

#                   PC1      PC2     r2 Pr(>r)    
#Log10Massg    -0.99271 -0.12054 0.9992  0.001 ***
#Invertebrates  0.28246 -0.95928 0.6975  0.001 ***
#Vertebrates   -0.08890 -0.99604 0.4891  0.001 ***
#Graze         -0.13096  0.99139 0.5832  0.001 ***
#Browse        -0.14771  0.98903 0.7115  0.001 ***



#Plot version with loadings
#pdf(width=10,height=7.5, file="Figure ExtraS283.pdf")
plot(origscore[,1:2], type="n",xlab=paste("PC1 (",eig1,"% of variance)",sep=""),ylab=paste("PC2 (",eig2,"% of variance)",sep=""))
text(origscore[,1:2], rownames(origscore), cex=0.3)
plot(ef, col="red")
#dev.off()



#Get native and extant species
natives <- origscore[,1:2]
modern <- origscore[allrange$Status=="Extant",1:2]

#Get the convex hull values
nativehull <- chull(natives)
nativehull <- c(nativehull,nativehull[1])

modernhull <- chull(modern)
modernhull <- c(modernhull,modernhull[1])


#Plot PCA Space with convex hulls
pdf(width=6,height=4.5, file="Figure S28.pdf")
par(mar=c(5.1,4.1,1,2.1))

plot(origscore[,1:2],type="n",xlab=paste("PC1 (",eig1,"% of variance)",sep=""),ylab=paste("PC2 (",eig2,"% of variance)",sep=""))

#Plot convex hulls
lines(natives[nativehull,1:2],col="#ff7777",lwd=3.4)
lines(modern[modernhull,1:2],col="lightskyblue",lwd=4.2)

#Plot points
points(origscore[allrange$Status=="Extant",1:2],col="dodgerblue2",pch=19)
points(origscore[allrange$Status=="Extinct",1:2],col="red1",pch=19)
#points(origscore[allrange$Status=="Introduced",1:2],col="green4",pch=19)

dev.off()



rm(list = ls())
######################################################

#SIMULATE TOTAL MAMMAL FUNCTIONAL SPACE

#Now to simulate what happens to functional diversity results with different mass cutoffs
allrange3 <- read.csv(file="Allrange All North American.csv", row.names=1)
dim(allrange3)#735 14
head(allrange3)
tail(allrange3)

#Take out all introduced species
allrange2 <- allrange3[allrange3$Status!="Introduced",]
dim(allrange2)#728 14

#And take out Homo sapiens and Bison bison because they arrived after 50,000 years.
allrange <- allrange2[allrange2$Binomial!="Homo sapiens" & allrange2$Binomial!="Bison bison" ,]
dim(allrange)#726 14

summary(allrange)
#That leaves us with 676 Extant species and 50 Extinct species with mean log10 mass of 2.0850


#Pull out the traits
colnames(allrange)
traits <- allrange[,c(9:13)]
traits

#Set up the weights.  Weighting so mass and diet each get half 
groupweight <- 1/2
weights <- c(groupweight,rep(groupweight/4,4))

	
#Spoof the dates
dates <- matrix(data=1, nrow=length(rownames(traits)), ncol=2)
colnames(dates) <- c("Late Pleistocene","Pre-Columbian")
rownames(dates) <- rownames(traits)
dates[allrange$Status=="Extinct",2] <- 0

head(traits)
head(dates)
dim(traits)#726 5
dim(dates)#726 2

#Set the number of iterations for the simulation
n=1000

source('Davis Functional Diversity Simulation Mass Cutoffs.R', chdir = F)

str(results)


rm(list = ls())
########################

#PLOT SIMULATION RESULTS

#Now plot the results of that simulation

require(ggplot2)
require(plyr)
require(reshape2)
require(grid)
require(data.table)

#Load up results
load("Functional Simulation Results Mass Cutoff 1000")
str(results)

data1 <- results$realdata1
simdf <- results$simdata1
gcutoff <- results$gcutoff
logcutoff <- results$logcutoff

head(simdf)



#Calculate the confidence intervals

#First have to melt the data frame
meltsim <- melt(simdf,id.vars=c("Cutoff","NumExtant","NumExtinct"))
head(meltsim)
colnames(meltsim)[4:5] <- c("Index","Value")
head(meltsim)
tail(meltsim)

#figure out medians and 95% confidence intervals for all the functional indices
indexsum <- ddply(meltsim,.(Cutoff, NumExtant, NumExtinct, Index), summarise,Median=quantile(Value, 0.5,na.rm=T),CIl95=quantile(Value,0.025,na.rm=T),CIu95=quantile(Value, 0.975,na.rm=T))
dim(indexsum)#24 7
indexsum

indices <- sort(unique(meltsim$Index))



###################

#Now prep the real data

data1


#Melt the prediction data
meltdata <- melt(data1,id.vars=c("Cutoff","NumExtant","NumExtinct"))
colnames(meltdata)[4:5] <- c("Index","Value")
meltdata

#And for labels
datalab <- data1[,1:3]
meltlabel <- melt(datalab, id.vars=c("Cutoff"))
colnames(meltlabel)[2:3] <- c("Variable","Value")
levels(meltlabel$Variable) <- list("Extinct species"="NumExtinct", "Extant species"="NumExtant")


meltlabel[meltlabel$Variable=="NumExtant", "Variable"]


#######################
#To automatically load in plot parameters
source("Davis Plot Parameters.R", chdir = F)


p <- ggplot(data=indexsum)+
#Version with just FD and FRic
#p <- ggplot(data=subset(indexsum, Index=="FD" | Index=="FRic"))+

#Plot null expecations
geom_crossbar(aes(x=Cutoff, ymin=CIl95, ymax=CIu95, y=Median, color=Index, fill=Index), alpha=0.15)+
#Plot the real results
geom_point(data=meltdata,aes(x=Cutoff,y=Value),color="black",size=2.5)+

#Version with just FD and FRic
#geom_crossbar(aes(x=Cutoff, ymin=CIl95, ymax=CIu95, y=Median, color=Index, fill=Index), alpha=0.15)+
#geom_point(data=subset(meltdata, Index=="FD" | Index=="FRic"),aes(x=Cutoff,y=Value),color="black",size=2.5)+

#Set up colors to match previous graphs
scale_color_manual(name="Index",values=c("#ff7f00","#377eb8","#4daf4a","#984ea3"))+
scale_fill_manual(name="Index",values=c("#ff7f00","#377eb8","#4daf4a","#984ea3"))+

facet_grid(Index ~., scales="free_y")+
scale_x_continuous(name="Mass cutoff", breaks=logcutoff, labels=gcutoff )+
scale_y_continuous(name="Pre-Columbian index value\nrelative to late Pleistocene value")+
theme(legend.position="none")+
#Remove labels
#theme(strip.background=element_blank(),strip.text=element_blank())+
simpleblack

#Make the data table that shows how many species are in each mass cutoff
data_table <- ggplot(meltlabel, aes(x=Cutoff, y = Variable, label=Value, color=Variable))+
geom_text(size = 3.5)+ 
theme_bw() +
scale_color_manual(values=c("Extant species"="dodgerblue2","Extinct species"="red1"))+
theme(panel.grid.major = element_blank(), legend.position = "none",panel.border = element_blank(), axis.text.x = element_blank(),axis.ticks = element_blank(),plot.margin = unit(c(0.25,3.5, 0,0.75), "line")) + 
xlab(NULL) + 
ylab(NULL)



#A little trick from https://learnr.wordpress.com/2009/04/29/ggplot2-labelling-data-series-and-adding-a-data-table/

Layout <- grid.layout(nrow = 2, ncol = 1, heights = unit(c(2,0.25), c("null", "null")))
     #grid.show.layout(Layout)
vplayout <- function(...) {
     grid.newpage()
     pushViewport(viewport(layout = Layout))
 }
subplot <- function(x, y) viewport(layout.pos.row = x,layout.pos.col = y)
mmplot <- function(a, b) {
     vplayout()
     print(a, vp = subplot(1, 1))
     print(b, vp = subplot(2, 1))
 }
 

pdf(width=6, height=4.5,file="Figure S29.pdf") 
 
mmplot(p, data_table)

dev.off()



######################################################


#Now plot functional distinctiveness based on mass cutoff

#Pull out the list and turn it into one big dataframe
distinctlist <- results$distinctlist
str(distinctlist)

disdf <- as.data.frame(rbindlist(distinctlist))
head(disdf)
tail(disdf)
disdf$Cutoffg <- factor(disdf$Cutoffg, levels=c("1 g","10 g","100 g","1 kg","10 kg","100 kg"))


ggplot(data=disdf, aes(x=Status, y=FDprel*100, fill=Status))+
geom_boxplot()+
facet_grid(.~Cutoffg)+
scale_fill_manual(name="Status",values=c("Extant"="lightskyblue","Extinct"="#ff7777"))+
scale_y_continuous(name="Functional distinctiveness")+
simpleblack+
theme(legend.position="none", axis.title.x=element_blank(), axis.text.x=element_text(size=8))


ggsave(file="Figure S30.pdf",width=6, height=4.5)








#####################################################
#####################################################
#END OF CODE
#####################################################
#####################################################
#
#              ___.-~"~-._   __....__
#            .'    `    \ ~"~        ``-.
#           /` _      )  `\              `\
#          /`  a)    /     |               `\
#         :`        /      |                 \
#    <`-._|`  .-.  (      /   .            `;\\
#     `-. `--'_.'-.;\___/'   .      .       | \\
#  _     /:--`     |        /     /        .'  \\
# ("\   /`/        |       '     '         /    :`;
# `\'\_/`/         .\     /`~`=-.:        /     ``
#   `._.'          /`\    |      `\      /(
#                 /  /\   |        `Y   /  \
#           jgs  J  /  Y  |         |  /`\  \
#               /  |   |  |         |  |  |  |
#              "---"  /___|        /___|  /__|
#                     '"""         '"""  '"""