#Davis Functional Diversity Random Simulation

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This is function file that will create n simulations of species going extinct in a random order to compare to actual results. 

#This corresponds to simfd5 in the author's own version control system



simfd5 <- function(traits=traits, weights=weights, n=n){

require(FD)	
require(cluster)
require(picante)
	
#Bookkeeping
traits <- as.matrix(traits)



alltaxa <- rownames(traits)
numtaxa <- dim(traits)[1]
numtraits <- dim(traits)[2]


	
#################################################

#Z transform the data so they are standardized so they have a sd of 1 and mean of 0

ndf <- apply(traits, 2, scale, center = T, scale = T)

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2
rownames(traits2) <- alltaxa

#################################################	
	#Make the distance matrix for the tree, don't want standardization because we already standardized and weighted
	
	dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
	#Now make the tree
	tree <- hclust(dist1,method="average")
	#plot(tree)
	
	#Run the tree through the Xtree function which breaks up all the branch lengths.
	#Turn into a phylo object
	ftree <- as.phylo(tree)
	#plot(ftree)

#################################################
#################################################



	#But we need to figure out how many PCO(PCA if euclidean) axes to keep following the methology of the Random average under permutation method from Peres-Neto, P. R., Jackson, D. A., & Somers, K. M. (2005). How many principal components? Stopping rules for determining the number of non-trivial axes revisited. Computational Statistics & Data Analysis, 49(4), 974–997. http://doi.org/10.1016/j.csda.2004.06.015

	#Need to randomize values within each trait and run a pca

	#Do the number of randomizations. Should use 1,000
	axreps <- 1000

	#Make a matrix for the eigen values to go in
	eigmat <- matrix(data=NA,nrow=axreps,ncol=numtraits)

	#Now do the simulated values
	for(ee in 1:axreps){
		randtraits <- traits2
	
		#Go through each column and randomize the values
		randtraits2 <- apply(randtraits,2,sample)
		#Check should be same summary statistics as traits
		#summary(traits)
		#summary(randtraits2)
	
		#Now do a pca
		pca1 <- prcomp(randtraits2,center=F,scale=F)
	
		#Put the eigenvalues=proportion of variance explained into the eigmatratix
		eigmat[ee,] <- summary(pca1)$importance[2,]
	
	}#end ee loop

	#Get the mean value of each eigenvalue
	meaneig <- colMeans(eigmat)


	#Now do a real pca
	pcareal <- prcomp(traits2,center=F,scale=F)

	#Pull out real eigenvalues
	realeig <- summary(pcareal)$importance[2,]
	#realeig-meaneig	

	#How many axes should we keep
	numaxes <- length(which(realeig>meaneig))

	#And how much variation do they explain
	propvar <- realeig[1:numaxes]

	#And what are those axes
	pctraits <- 	pcareal$x[,1:numaxes]
	rownames(pctraits) <- alltaxa
	#And make a distance matrix of them
	#pcdist <- daisy(pctraits, metric="euclidean",stand=F) 


#################################################

	#Make the matrix that will be filled up with each iteration of the functional analysis
	data1 <- as.data.frame(matrix(data=NA, nrow=numtaxa,ncol=7,dimnames=list(c(1:numtaxa),c("SpeciesRichness","RelRich", "FD", "FRic","FDis","RaoQ","Iteration"))))

	#And the list that will hold all of the data1's
	simlist <- list()

#################################################

#Actual calculation of functional diversity indices
	
###############################################		

for(z in 1:n){
		
	#To do the randomized extinction
	#Create a vector of random LADs  
	 allrange<- as.vector(sample(0:(numtaxa-1), replace=F,numtaxa))
	
	
	#Generate a random therematrix
	#Make the occurence matrix
	therematrix <- matrix(data=1, nrow=numtaxa, ncol=numtaxa, dimnames=list(c(1:(numtaxa)),alltaxa))

	
	#Now fill it up
	for(i in 1:numtaxa){
		rownumber <- allrange[i]
		
		if(rownumber!=0){
		
		therematrix[1:rownumber,i] <- rep(0,allrange[i])
		}
		
	}#End i loop
	
	#to check
	#apply(therematrix,1,sum)
	
	
	
###############################################		
		
	
	#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure

	FD1 <- pd(samp=therematrix, tree=ftree, include.root=F)
	
	#Put it in the results matrix
	data1$SpeciesRichness <- FD1$SR
	
	#Standardize FD
	data1$FD <- FD1$PD/max(FD1$PD,na.rm=T)

	
#################################################


#FRic

	#Create a vector for the absolute values to go in
	absvol <- rep(NA,numtaxa)
	
	#To ensure we don't measure any communities we cant measure.  FDis can be measured on smaller communities but we won't be looking at any that small in real data anyway
	startpoint <- 10
	#startpoint <- min(which(rowSums(therematrix)>numaxes))
	
	
	#Loop through each timebin
	for(bb in startpoint:numtaxa){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(therematrix[bb,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
		
		####FRic part
		#Get the total hull volume for the species in this bin
		#Use FA for volume and Fx for vertices
		conhull <- convhulln(thesetraits, options=c("FA"))
	
		#First get the volume 
		absvol[bb] <- conhull$vol
		
		
		####FDis part
		#Calculate centroid and attach it the traits
		thiscent <- colMeans(thesetraits)
		thesetraits2 <- rbind(thesetraits,thiscent)
		
		#Now calculate distance to centroid
		totcentroiddist <- as.matrix(dist(thesetraits2, method="euclidean",diag=T,upper=T))
		#Remove the zero
		centdist <- totcentroiddist["thiscent",1:numthisbintaxa]
		
		#Now put the values into the results matrix
		data1$FDis[bb] <- mean(centdist)
	
		}#End bb loop 
	
		#Now make the the convex hull volumes relative
		data1$FRic <- absvol/absvol[numtaxa]


#################################################

	#Calculate Rao's Quadratic entropy
	#First make the right distance matrix
	pcdist <- daisy(pctraits, metric="euclidean",stand=F) 
	
	RaoQ <- divc(as.data.frame(t(therematrix)),pcdist,scale=F)
	colnames(RaoQ) <- "RaoQ"
	data1$RaoQ <- c(as.matrix(RaoQ))	
		 
	
#################################################
#################################################
	
	#Fill in relative species richness
	data1$RelRich <- data1$SpeciesRichness/data1$SpeciesRichness[numtaxa]
	
	#And what iteration it was
	data1[,"Iteration"] <- z
	
	

	
	#head(data1)
	#tail(data1)
	
	
	#Now put those results into the result list
	simlist[[z]] <- data1
		
		
}#End z (n) loop
	
	
	#Make a list to return results
	simresults <- list(traits2=traits2,pctraits=pctraits,weights=weights, numaxes=numaxes, simlist=simlist)
	
	return(simresults)
	
	}#End function