#Davis Functional Diversity Simulation Mass Cutoffs

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This is a function file that will simulate how functional diversity values change depending on where the mass cutoff is for excluding species from analysis


require(geometry)	
require(cluster)
require(picante)
require(ade4)




#Make a copy of the original traits and dates
otraits <- traits
odates <- dates

#Figure out mass cutoffs in log10 grams
#Could actually go to a cutoff of 1,000 kg but then only extinct species would be left so Functional diversity values would be meaningless because the community would have zero species
logcutoff <- c(0:5)
numcutoff <- length(logcutoff)
#And in grams
gcutoff <- c("1 g","10 g","100 g","1 kg","10 kg","100 kg")
 
 
#Create the dataframe that will hold all of the actual results
realdata1 <- as.data.frame(matrix(data=NA, ncol=7, nrow=numcutoff))
colnames(realdata1) <-c("Cutoff","NumExtant","NumExtinct","FD","FRic","FDis","RaoQ")
realdata1$Cutoff <- logcutoff



#And the dataframe that will hold all of the simulation data for random extinctions
simdata1 <- as.data.frame(matrix(data=NA, ncol=7, nrow=numcutoff*n))
colnames(simdata1) <-c("Cutoff","NumExtant","NumExtinct","FD","FRic","FDis","RaoQ")
simdata1$Cutoff <- rep(logcutoff,each=n)




#The vector that will show how many PCA axes were used
totnumaxes <- c()
totpropvar <- c()



#And the list that will hold hold functional distinctiveness values
distinctlist <- list()

########################################################


#Set the overall loop that will step through different mass cutoffs

for(mm in 1:numcutoff){
	
	#Cut out all species that are below this mass threshold
	traits <- otraits[otraits[,"Log10Massg"]>=logcutoff[mm],]
	#dim(traits)
	dates <- dates[rownames(dates) %in% rownames(traits),]
	#dim(dates)
	
	numextant <- sum(dates[,2])
	numextinct <- sum(dates[,1])-numextant
	
	realdata1[mm,"NumExtant"] <- numextant
	realdata1[mm,"NumExtinct"] <- numextinct

	simdata1[c((((mm-1)*n)+1): (mm*n)),"NumExtant"] <- numextant
	simdata1[c((((mm-1)*n)+1): (mm*n)),"NumExtinct"] <- numextinct

	##################################################
	
	#Now create the functional space
	
	#Bookkeeping
	traits <- as.matrix(traits)



	alltaxa <- rownames(traits)
	numtaxa <- dim(traits)[1]
	numtraits <- dim(traits)[2]

	timesteps <- dim(dates)[2]
	
	#Set the real occurence matrix
	realtherematrix <- t(dates)
	
	#################################################
	
	#Z transform the data so they are standardized so they have a sd of 1 and mean of 0
	
	ndf <- apply(traits, 2, scale, center = T, scale = T)
	
	#Check
	#apply(ndf,2,sd)
	#apply(ndf,2,mean)
	
	#Now apply weighting to the traits
	#This transposes the matrix so that columns are rows and then transposes it back again
	ndf2 <- t(t(ndf)*weights)
	
	#Check the standard deviations.  They were 1 before and they should be the weights now
	#summary(ndf2)
	#apply(ndf2,2,sd)
	
	traits2 <- ndf2
	rownames(traits2) <- alltaxa
	
#################################################	
	#Make the distance matrix for the tree, don't want standardization because we already standardized and weighted
	
	dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
	#Now make the tree
	tree <- hclust(dist1,method="average")
	#plot(tree)
	
	#Run the tree through the Xtree function which breaks up all the branch lengths.
	#Turn into a phylo object
	ftree <- as.phylo(tree)
	#plot(ftree)




#################################################

	#We need to figure out how many PCO(PCA if euclidean) axes to keep following the methology of the Random average under permutation method from Peres-Neto, P. R., Jackson, D. A., & Somers, K. M. (2005). How many principal components? Stopping rules for determining the number of non-trivial axes revisited. Computational Statistics & Data Analysis, 49(4), 974–997. http://doi.org/10.1016/j.csda.2004.06.015

	#Need to randomize values within each trait and run a pca

	#Do the number of randomizations. Should use 1,000
	axreps <- 1000

	#Make a matrix for the eigen values to go in
	eigmat <- matrix(data=NA,nrow=axreps,ncol=numtraits)

	#Now do the simulated values
	for(ee in 1:axreps){
		randtraits <- traits2
	
		#Go through each column and randomize the values
		randtraits2 <- apply(randtraits,2,sample)
		#Check should be same summary statistics as traits
		#summary(traits)
		#summary(randtraits2)
	
		#Now do a pca
		pca1 <- prcomp(randtraits2,center=F,scale=F)
	
		#Put the eigenvalues=proportion of variance explained into the eigmatratix
		eigmat[ee,] <- summary(pca1)$importance[2,]
	
	}#end ee loop

	#Get the mean value of each eigenvalue
	meaneig <- colMeans(eigmat)


	#Now do a real pca
	pcareal <- prcomp(traits2,center=F,scale=F)

	#Pull out real eigenvalues
	realeig <- summary(pcareal)$importance[2,]
	#realeig-meaneig	

	#How many axes should we keep
	numaxes <- length(which(realeig>meaneig))
	totnumaxes[mm] <- numaxes


	#And how much variation do they explain
	propvar <- sum(realeig[1:numaxes])
	totpropvar[mm] <- propvar

	#And what are those axes
	pctraits <- 	pcareal$x[,1:numaxes]
	rownames(pctraits) <- alltaxa
	

###############################################		
		
	
	#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure

	FD1 <- pd(samp=realtherematrix, tree=ftree, include.root=F)
	
	
	#Standardize FD
	realdata1[mm,"FD"] <- (FD1$PD/max(FD1$PD,na.rm=T))[2]
	
	
###################################################

#Get functional distinctiveness for each species
FDp <- evol.distinct(ftree,type="fair.proportion",scale=F)
	colnames(FDp) <- c("Labels","FDp")
	#Add a relative branch length, don't multiply by 100 to keep consistant with Uniqueness
	FDp$FDprel <- (FDp$FDp/sum(FDp$FDp))
	#sum(FDp$FDprel)#Should equal 1
	#Add in columns with mass cutoff 
	FDp$Cutoff <- logcutoff[mm]
	FDp$Cutoffg <- gcutoff[mm]
	#And whether species are extinct or not
	FDp$Status <- "Extant"
	FDp$Status[dates[,2]==0] <- "Extinct"
	FDp2 <- FDp[,-2]

	#Put into the list
	distinctlist[[mm]] <- FDp2


#################################################

#Multidimensional space indices


#FRic

	#Create a matrix for the absolute values to go into before standardization
	absval <- as.data.frame(matrix(data=NA, nrow=timesteps,ncol=3))
	colnames(absval) <- c("FRic","FDis","RaoQ")
	rownames(absval) <- colnames(dates)
	
	
	#Loop through each timebin
	for(bb in 1:timesteps){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(realtherematrix[bb,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
		
		####FRic part
		#Get the total hull volume for the species in this bin
		#Use FA for volume and Fx for vertices
		conhull <- convhulln(thesetraits, options=c("FA"))
	
		#First get the volume 
		absval[bb, "FRic"] <- conhull$vol
		
		
		####FDis part
		#Calculate centroid and attach it to the traits
		thiscent <- colMeans(thesetraits)
		thesetraits2 <- rbind(thesetraits,thiscent)
		
		#Now calculate distance to centroid
		totcentroiddist <- as.matrix(dist(thesetraits2, method="euclidean",diag=T,upper=T))
		#Remove the zero
		centdist <- totcentroiddist["thiscent",1:numthisbintaxa]
		
		#Now put the values into the results matrix
		absval[bb,"FDis"] <- mean(centdist)
	
	
		}#End bb loop 
	
		


#################################################

	#Calculate Rao's Quadratic entropy
	#First make the right distance matrix
	pcdist <- daisy(pctraits, metric="euclidean",stand=F) 
	
	RaoQ <- divc(as.data.frame(t(realtherematrix)),pcdist,scale=F)
	absval$RaoQ <- c(t(RaoQ))	
		
	#Now make the functional values relative to a Late Pleistocene starting point
	realdata1[mm, 5:7]<- absval[timesteps,]/absval[1,]	 

#################################################
#################################################
#################################################
#################################################
	
	
	#Now start simulated data that will have random extinctions
	#Need to make a new therematrix with n iterations of random extinctions
	therematrix <- matrix(data=1,nrow=n,ncol=numtaxa)
	colnames(therematrix) <- colnames(realtherematrix)
	rownames(therematrix) <- paste("Iteration_",1:n,sep="")
	
	
	#Now create the random extinctions
	for(rr in 1:n){
		#Go through each row
		therematrix[rr,sample(numtaxa,size=realdata1[mm,"NumExtinct"], replace=F)] <- 0
		
	}#End create random extincitons loop
	
	#rowSums(therematrix)
	
	###############################################		
		
	#Now calculate Functional Diversity
	
	#Simulated Tree based FD 

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure

	simFD1 <- pd(samp=therematrix, tree=ftree, include.root=F)
	
	
	#Standardize FD
	simdata1[c((((mm-1)*n)+1): (mm*n)),"FD"] <- simFD1$PD/max(FD1$PD,na.rm=T)
	
	
	


#################################################

#Simulated Multidimensional space indices


#FRic

	#Create a matrix for the absolute values to go into before standardization
	simabsval <- as.data.frame(matrix(data=NA, nrow=n,ncol=3))
	colnames(simabsval) <- c("FRic","FDis","RaoQ")
	rownames(simabsval) <- rownames(therematrix)	
	
	

	
	
	#Loop through each iteration
	for(dd in 1:n){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(therematrix[dd,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
		
		####FRic part
		#Get the total hull volume for the species in this bin
		#Use FA for volume and Fx for vertices
		try(conhull <- convhulln(thesetraits, options=c("FA")), F)
	
		#First get the volume 
		#If the convhulln snags, this value will be the same as the iteration before it
		simabsval[dd, "FRic"] <- conhull$vol
		
		
		####FDis part
		#Calculate centroid and attach it the traits
		thiscent <- colMeans(thesetraits)
		thesetraits2 <- rbind(thesetraits,thiscent)
		
		#Now calculate distance to centroid
		totcentroiddist <- as.matrix(dist(thesetraits2, method="euclidean",diag=T,upper=T))
		#Remove the zero
		centdist <- totcentroiddist["thiscent",1:numthisbintaxa]
		
		#Now put the values into the simulated absolute values matrix
		simabsval[dd,"FDis"] <- mean(centdist)
	
	
		}#End dd loop 
	
		


#################################################

	#Calculate Rao's Quadratic entropy
	#First make the right distance matrix
	pcdist <- daisy(pctraits, metric="euclidean",stand=F) 
	
	RaoQ <- divc(as.data.frame(t(therematrix)),pcdist,scale=F)	
	simabsval$RaoQ <- c(t(RaoQ))	
		
	#Now make the functional values relative to a Late Pleistocene starting point
	simdata1[c((((mm-1)*n)+1): (mm*n)), 5:7] <- simabsval/c(as.vector(absval[1,]))
	
#################################################
	



}#End mm mass cutoff loop
 
 
 
 #Save results
 results <- list(realdata1=realdata1, simdata1=simdata1, totnumaxes=totnumaxes, totpropvar=totpropvar, n=n, traits=otraits, dates=odates, logcutoff=logcutoff, numcutoff=numcutoff, gcutoff=gcutoff, distinctlist=distinctlist)

 
 save(results, file=paste("Functional Simulation Results Mass Cutoff",n))