#Davis Functional Diversity Simulation

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate

#This is function file that will create n simulations of species going extinct in a random order to compare to actual results. 

#This null model fixes human and domestic animal introductions but keeps all extinctions random

#This corresponds to simfd6 in the author's own version control system




simfd6 <- function(traits=traits, weights=weights, realdata1=realdata1,dates=dates,n=n){

require(FD)	
require(cluster)
require(picante)


	
#Bookkeeping
traits <- as.matrix(traits)



alltaxa <- rownames(traits)
numtaxa <- dim(traits)[1]
numtraits <- dim(traits)[2]

timesteps <- length(realdata1$Date)
	
#################################################

#Z transform the data so they are standardized so they have a sd of 1 and mean of 0

ndf <- apply(traits, 2, scale, center = T, scale = T)

#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits
#This transposes the matrix so that columns are rows and then transposes it back again
ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2
rownames(traits2) <- alltaxa

#################################################	
	#Make the distance matrix for the tree, don't want standardization because we already standardized and weighted
	
	dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	
	#Now make the tree
	tree <- hclust(dist1,method="average")
	#plot(tree)
	
	#Run the tree through the Xtree function which breaks up all the branch lengths.
	#Turn into a phylo object
	ftree <- as.phylo(tree)
	#plot(ftree)

#################################################
#################################################



	#But we need to figure out how many PCO(PCA if euclidean) axes to keep following the methology of the Random average under permutation method from Peres-Neto, P. R., Jackson, D. A., & Somers, K. M. (2005). How many principal components? Stopping rules for determining the number of non-trivial axes revisited. Computational Statistics & Data Analysis, 49(4), 974–997. http://doi.org/10.1016/j.csda.2004.06.015

	#Need to randomize values within each trait and run a pca

	#Do the number of randomizations. Should use 1,000
	axreps <- 1000

	#Make a matrix for the eigen values to go in
	eigmat <- matrix(data=NA,nrow=axreps,ncol=numtraits)

	#Now do the simulated values
	for(ee in 1:axreps){
		randtraits <- traits2
	
		#Go through each column and randomize the values
		randtraits2 <- apply(randtraits,2,sample)
		#Check should be same summary statistics as traits
		#summary(traits)
		#summary(randtraits2)
	
		#Now do a pca
		pca1 <- prcomp(randtraits2,center=F,scale=F)
	
		#Put the eigenvalues=proportion of variance explained into the eigmatratix
		eigmat[ee,] <- summary(pca1)$importance[2,]
	
	}#end ee loop

	#Get the mean value of each eigenvalue
	meaneig <- colMeans(eigmat)


	#Now do a real pca
	pcareal <- prcomp(traits2,center=F,scale=F)

	#Pull out real eigenvalues
	realeig <- summary(pcareal)$importance[2,]
	#realeig-meaneig	

	#How many axes should we keep
	numaxes <- length(which(realeig>meaneig))

	#And how much variation do they explain
	propvar <- realeig[1:numaxes]

	#And what are those axes
	pctraits <- 	pcareal$x[,1:numaxes]
	rownames(pctraits) <- alltaxa
	#And make a distance matrix of them
	#pcdist <- daisy(pctraits, metric="euclidean",stand=F) 


#################################################

	#Make the matrix that will be filled up with each iteration of the functional analysis
	data1 <- as.data.frame(matrix(data=NA, nrow=timesteps,ncol=22,dimnames=list(c(realdata1$Date),c("SpeciesRichness","RelRich", "FD", "FRic","FDis","RaoQ","Iteration","Bin","Date","FDsim","FRicsim","Sorsim","Log10Massg","Invertebrates","Vertebrates","Graze","Browse","Running","Digging","Climbing","Swimming","Sociality"    ))))
	
	data1$Date <- realdata1$Date
	data1$Bin <- realdata1$Bin
	

	


	#And the list that will hold all of the data1's
	simlist <- list()

###############################
#Need to do some book keeping for the random extinction orders
#Pull out introduced species and add humans and dogs
#Paleoindians and dogs came in first, then Europeans and dogs and stock animals
introducedfirst <- c(alltaxa[allrange$Status=="Introduced" & allrange$FAD>10000],"Homo_sapiens")
introducedsecond <- unique(c(alltaxa[allrange$Status=="Introduced"],introducedfirst))


#Now all the other species
#Note that this includes Bison bison which is native but is assumed to evolve in situ in this data set. 
allnative <- alltaxa[!alltaxa %in% introducedsecond]
length(allnative)#86

#Times that introductions happen
firsttime <- allrange[allrange$Labels=="Homo_sapiens","FAD"]
secondtime <- allrange[allrange$Labels=="Sus_scrofa","FAD"]


############################################################
#Need to have the real occurence matrix to calculate functional similarity on
#Make the occurence matrix
#Need to add one row so that there is a step with all species and species iterations in the matrix

	realtherematrix <- matrix(data=0, nrow=timesteps, ncol=numtaxa, dimnames=list(c(realdata1$Date),rownames(dates)))
	#dim(therematrix)#53 94


	#Now fill it up
	#For each species
	for(i in 1:numtaxa){
	
	thistaxa <- alltaxa[i]
	
	#Take out lads and fads
	FAD <- dates[i,1]
	LAD <- dates[i,2]
	
		#For each timestep
		for(j in 1:timesteps){
	
			#And fill up only that taxa's column 
			realtherematrix[j,thistaxa] <- 	ifelse((FAD>=realdata1$Date[j] & LAD<realdata1$Date[j+1]),1,0)


			
		}#End j loop
	}#End i loop

	#Insert all 1's in the last row so that there a fake timeslice with all species 
	
	realtherematrix[timesteps,] <- 1
	
	
	###############
	#Calculate real trait means so they can be compared to null trait means
	realtraitmeans <- matrix(data=NA, ncol=numtraits, nrow=timesteps, dimnames=list(realdata1$Date,colnames(traits)))
	
	for(ff in 1:timesteps){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(realtherematrix[ff,]==1)]
		
		#Get their traits
		thesetraitsorig <- traits[thisbintaxa,]
		realtraitmeans[ff,colnames(traits)] <- colMeans(thesetraitsorig)
	
		}#End ff loop 

	
	
	
	
#################################################

#Actual calculation of functional diversity indices
	
###############################################		

for(z in 1:n){
		
	#To do the randomized extinction with set introductions
	
	
	#Generate a random therematrix
	#Make the occurence matrix
	therematrix <- matrix(data=0, nrow=timesteps, ncol=numtaxa, dimnames=list(allrange$Date,alltaxa))

	#So start at the earliest bin, and select as many species as you have in the real dataset and continue on through each bin
	#Note that this could select Bison bison in the first bin even though it is assumed to evolve in situ later
	#Make a list of bins
	prevbinlist <- list()
	prevbinlist[[(timesteps)]] <- allnative
	
	#Now fill it up
	for(i in (timesteps-1):1){
		#Pull out the previous bin
		previousbintaxa <- prevbinlist[[i+1]]
		
		
		#First first colonization of humans
		if(firsttime>=data1$Date[i] & firsttime<data1$Date[i+1]){
			#Have to figure out right number of taxa to keep
			preintrotaxa <-sample(previousbintaxa,size=realdata1$SpeciesRichness[i]-length(introducedfirst),replace=F)
				
			thisbintaxa <-  c(preintrotaxa,introducedfirst)
			#Mark those species as present
			therematrix[i,thisbintaxa] <- 1
		
			prevbinlist[[i]] <- thisbintaxa
			
			} else if(secondtime>=data1$Date[i] & secondtime<data1$Date[i+1]){
			#Have to figure out right number of taxa to keep and humans or dogs may not have gone extinct 
			
			numtake <- length(introducedsecond)-sum(introducedfirst %in% previousbintaxa)
			
			preintrotaxa <-sample(previousbintaxa,size=realdata1$SpeciesRichness[i]-numtake,replace=F)
				
			thisbintaxa <-  unique(c(preintrotaxa,introducedsecond))
			#Mark those species as present
			therematrix[i,thisbintaxa] <- 1
		
			prevbinlist[[i]] <- thisbintaxa
			
			} else {
		
		thisbintaxa <- sample(previousbintaxa,size=realdata1$SpeciesRichness[i],replace=F)
		
		#Mark those species as present
		therematrix[i,thisbintaxa] <- 1
		
		prevbinlist[[i]] <- thisbintaxa
		
		}}#End i loop
		
		#Need to add all 1's in the earliest bin as it is a fake total bin
		therematrix[timesteps,] <- 1
	
	#to check
	#cbind(realdata1$SpeciesRichness,apply(therematrix,1,sum))
	
	
	
###############################################		
		
	
	#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure

	FD1 <- pd(samp=therematrix, tree=ftree, include.root=F)
	
	#Put it in the results matrix
	data1$SpeciesRichness <- FD1$SR
	
	#Standardize FD
	data1$FD <- FD1$PD/max(FD1$PD,na.rm=T)
	
	#And calculate tree based functional similarity.Loop through each bin and get comparison
	for(q in 1:timesteps){
		comptherematrix <- rbind(therematrix[q,],realtherematrix[q,])
	data1[q,"FDsim"] <- phylosor(comptherematrix,ftree)
	
	#And calculate Sørensen similarity Index too to get taxic similarity
	data1[q,"Sorsim"] <- 1-vegdist(comptherematrix,method="bray")
	
	}#End q loop
	
	#plot(data1$Sorsim,data1$FDsim)
	

#################################################


#FRic

	#Create a vector for the absolute values to go in
	absvol <- rep(NA,timesteps)
	
	
	#Loop through each timebin
	for(bb in 1:timesteps){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(therematrix[bb,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
		
		####FRic part
		#Get the total hull volume for the species in this bin
		#Use FA for volume and Fx for vertices
		conhull <- convhulln(thesetraits, options=c("FA"))
	
		#First get the volume 
		absvol[bb] <- conhull$vol
		
		
		####FDis part
		#Calculate centroid and attach it the traits
		thiscent <- colMeans(thesetraits)
		thesetraits2 <- rbind(thesetraits,thiscent)
		
		#Now calculate distance to centroid
		totcentroiddist <- as.matrix(dist(thesetraits2, method="euclidean",diag=T,upper=T))
		#Remove the zero
		centdist <- totcentroiddist["thiscent",1:numthisbintaxa]
		
		#Now put the values into the results matrix
		data1$FDis[bb] <- mean(centdist)
	
	
		####FRicsim Volume overlap
		#Not going to calculate in this version
	
		####Trait means part
		thesetraitsorig <- traits[thisbintaxa,]
		data1[bb,colnames(traits)] <- colMeans(thesetraitsorig)
	
		}#End bb loop 
	
		#Now make the the convex hull volumes relative
		data1$FRic <- absvol/absvol[timesteps]


#################################################

	#Calculate Rao's Quadratic entropy
	#First make the right distance matrix
	pcdist <- daisy(pctraits, metric="euclidean",stand=F) 
	
	RaoQ <- divc(as.data.frame(t(therematrix)),pcdist,scale=F)
	colnames(RaoQ) <- "RaoQ"
	data1$RaoQ <- c(as.matrix(RaoQ))	
		 
	
#################################################
#################################################
	
	#Fill in relative species richness
	data1$RelRich <- data1$SpeciesRichness/data1$SpeciesRichness[timesteps]
	
	#And what iteration it was
	data1[,"Iteration"] <- z
	
	

	
	#head(data1)
	#tail(data1)
	
	
	#Now put those results into the result list
	simlist[[z]] <- data1
		
		
}#End z (n) loop
	
	
	#Make a list to return results
	simresults <- list(traits2=traits2,pctraits=pctraits,weights=weights, numaxes=numaxes, simlist=simlist,realtraitmeans=realtraitmeans)
	
	return(simresults)
	
	}#End function