#Davis Functional Diversity

#Written by Matt Davis matthew.davis@yale.edu
#Please acknowledge or cite where appropriate


#This is a simplified function that will calculate FD, (and FRic, FDis and RaoQ on reduced trait axes) over time. 

#It takes as input traits, dates, weights, and a datatype argument

#This corresponds to quickfricfdladfad12 in the author's own version control system


quickfricfdladfad12 <- function(traits=traits, dates=dates, weights=weights, datatype=datatype){

require(geometry)	
require(cluster)
require(picante)
require(ade4)
	
#Bookkeeping
traits <- as.matrix(traits)

alltaxa <- rownames(traits)
numtaxa <- dim(traits)[1]
numtraits <- dim(traits)[2]
	
#Save a copy of traits before standardization
origtraits <- traits	
	
	
#######################################################

#Need to standardize the trait matrix here

#Z transform the data so they are standardized 
#Traits transformed so they have a sd of 1 and mean of 0

	df <- traits
	ndf <- df
for(i in 1:dim(df)[1]){
	for(j in 1:dim(df)[2]){
		ndf[i,j] <- (df[i,j]-mean(df[,j], na.rm=T))/sd(df[,j], na.rm=T)
	}#End j loop
	}#End i loop




#Check
#apply(ndf,2,sd)
#apply(ndf,2,mean)

#Now apply weighting to the traits because it will not be done later
#This transposes the matrix so that columns are rows and then transposes it back again

ndf2 <- t(t(ndf)*weights)

#Check the standard deviations.  They were 1 before and they should be the weights now
#summary(ndf2)
#apply(ndf2,2,sd)

traits2 <- ndf2

#################################################	
	#Make the distance matrix, don't want standardization because we already standardized and weighted
	
	dist1 <- daisy(traits2, metric="euclidean",stand=F) 
	
	#Now make the tree
	tree <- hclust(dist1,method="average")
	#plot(tree)
	
	#Turn into a phylo object
	ftree <- as.phylo(tree)
	
#################################################



	#But we need to figure out how many PCO(PCA if euclidean) axes to keep following the methology of the Random average under permutation method from Peres-Neto, P. R., Jackson, D. A., & Somers, K. M. (2005). How many principal components? Stopping rules for determining the number of non-trivial axes revisited. Computational Statistics & Data Analysis, 49(4), 974–997. http://doi.org/10.1016/j.csda.2004.06.015

	#Need to randomize values within each trait and run a pca

	#Do the number of randomizations. Should use 1,000
	axreps <- 1000

	#Make a matrix for the eigen values to go in
	eigmat <- matrix(data=NA,nrow=axreps,ncol=numtraits)

	#Now do the simulated values
	for(ee in 1:axreps){
		randtraits <- traits2
	
		#Go through each column and randomize the values
		randtraits2 <- apply(randtraits,2,sample)
		#Check should be same summary statistics as traits
		#summary(traits)
		#summary(randtraits2)
	
		#Now do a pca
		pca1 <- prcomp(randtraits2,center=F,scale=F)
	
		#Put the eigenvalues=proportion of variance explained into the eigmatratix
		eigmat[ee,] <- summary(pca1)$importance[2,]
	
	}#end ee loop

	#Get the mean value of each eigenvalue
	meaneig <- colMeans(eigmat)


	#Now do a real pca
	pcareal <- prcomp(traits2,center=F,scale=F)

	#Pull out real eigenvalues
	realeig <- summary(pcareal)$importance[2,]
	#realeig-meaneig	

	#How many axes should we keep
	numaxestokeep <- length(which(realeig>meaneig))

	#And how much variation do they explain
	proportion.variance.explained <- realeig[1:numaxestokeep]

	#And what are those axes
	pctraits <- 	pcareal$x[,1:numaxestokeep]
	
	#And make a distance matrix of them
	pcdist <- daisy(pctraits, metric="euclidean",stand=F) 


#################################################

#Create species timebin occurence matrix

	#For equal time bins including possible future extinctions
	inoutdates1 <-seq(-2000,50000,by=1000)
	
	timesteps <- length(inoutdates1)
	
	bins <- inoutdates1
	
	#Make the occurence matrix
	#Need to add one row so that there is a step with all species and species iterations in the matrix

	therematrix <- matrix(data=0, nrow=timesteps, ncol=numtaxa, dimnames=list(c(inoutdates1),rownames(dates)))
	#dim(therematrix)#53 94


	#Now fill it up
	#For each species
	for(i in 1:numtaxa){
	
	thistaxa <- alltaxa[i]
	
	#Take out lads and fads
	FAD <- dates[i,1]
	LAD <- dates[i,2]
	
		#For each timestep
		for(j in 1:timesteps){
	
			#And fill up only that taxa's column 
			therematrix[j,thistaxa] <- 	ifelse((FAD>=bins[j] & LAD<bins[j+1]),1,0)


			
		}#End j loop
	}#End i loop

	#Insert all 1's in the last row so that there a fake timeslice with all species 
	
	therematrix[timesteps,] <- 1
	

#################################################
#################################################

#Calculate Functional Diversity


	#Make a matrix of the results
	 data1<- as.data.frame(matrix(data=NA, nrow=timesteps, ncol=7, dimnames=list(rownames(therematrix),c("SpeciesRichness", "FD", "FRic","FDis","RaoQ","Bin","Date"))))

	data1$Date <- bins
	data1$Bin <- c(1:timesteps)
	
#################################################

#Tree based FD

	#Use the phylogenetic diversity function in picante.  Don't include root to follow Petchey and Gaston's modified FD measure
	FD1 <- pd(therematrix, ftree, include.root=F)
	
	#Put it in the results matrix
	data1$SpeciesRichness <- FD1$SR
	
	#Standardize FD
	data1$FD <- FD1$PD/max(FD1$PD)
	
	
#################################################


#FDis and #FRic

#Create a vector for the absolute FRic values to go in
	absvol <- rep(NA,timesteps)

	
	#Loop through each timebin
	for(bb in 1:timesteps){
		
		#Get the species in this bin
		thisbintaxa <- alltaxa[which(therematrix[bb,]==1)]
		numthisbintaxa <- length(thisbintaxa)
	
		#Get their traits
		thesetraits <- pctraits[thisbintaxa,]
	
		#Calculate centroid and attach it the traits
		thiscent <- colMeans(thesetraits)
		thesetraits2 <- rbind(thesetraits,thiscent)
		
		#Now calculate distance to centroid
		totcentroiddist <- as.matrix(dist(thesetraits2, method="euclidean",diag=T,upper=T))
		#Remove the zero
		centdist <- totcentroiddist["thiscent",1:numthisbintaxa]
		
		#Now put the values into the results matrix
		data1$FDis[bb] <- mean(centdist)
	
	
		####And calculate FRic 

		#Use FA for volume and Fx for vertices
		conhull <- convhulln(thesetraits, options=c("FA"))
		
		#First get the volume 
		absvol[bb] <- conhull$vol	
	
		}#End bb loop 
	
		#Now make the the convex hull volumes relative
		data1$FRic <- absvol/absvol[timesteps]


#################################################

	#Calculate Rao's Quadratic entropy
	RaoQ <- divc(as.data.frame(t(therematrix)),pcdist,scale=F)
	colnames(RaoQ) <- "RaoQ"
	data1$RaoQ <- c(as.matrix(RaoQ))
	
#################################################
#################################################


#Assemble results and print them
results <- list(datatype=datatype,data1=data1,traits=origtraits,dates=dates,weights=weights,numaxes=numaxestokeep,propvar=proportion.variance.explained,pctraits=pctraits)	


return(results)	
		
		
	}#End function




